.TH Buffer 3 "15 November 2002"
.SH NAME
Buffer, BufferCreate, BufferGets, BufferFgets \- operations on strings of undeterminate length
.SH SYNOPSIS
.PP
.nf
.ft B
#include "Buffer.h"
.ft
.fi
.PP
.nf
.ft B
Buffer *BufferCreate(s, i)
	int s;
	int i;
.ft
.fi
.PP
.nf
.ft B
Buffer *BufferFromString(s)
	char *s;
.ft
.fi
.PP
.nf
.ft B
void BufferFree(b);
	Buffer *b;
.ft
.fi
.PP
.nf
.ft B
char *BufferReset(b)
	Buffer *b;
.ft
.fi
.PP
.nf
.ft B
char* BufferString(b);
	Buffer *b;
.ft
.fi
.PP
.nf
.ft B
char *BufferSave(b);
	Buffer *b;
.ft
.fi
.PP
.nf
.ft B
char *BufferCopy(b1, b2)
	Buffer *b1;
	Buffer *b2;
.ft
.fi
.PP
.nf
.ft B
char *BufferStrcpy(b, s)
	Buffer *b;
	char *s;
.ft
.fi
.PP
.nf
.ft B
char *BufferStrncpy(b, s, l)
	Buffer *b;
	char *s;
	int l;
.ft
.fi
.PP
.nf
.ft B
char *BufferCat(b1, b2)
	Buffer *b1;
	Buffer *b2;
.ft
.fi
.PP
.nf
.ft B
Buffer *BufferClone(b1)
	Buffer *b1;
.ft
.fi
.PP
.nf
.ft B
char *BufferStrcat(b, s)
	Buffer *b;
	char *s;
.ft
.fi
.PP
.nf
.ft B
char *BufferStrncat(b, s, n)
	Buffer *b;
	char *s;
	int n;
.ft
.fi
.PP
.nf
.ft B
char *BufferFgets(b,s)
	Buffer *b;
	FILE *s;
.ft
.fi
.PP
.nf
.ft B
char *BufferGets(b)
	Buffer *b;
.ft
.fi
.PP
.nf
.ft B
int BufferSize(b)
	Buffer *b;
.ft
.fi
.PP
.nf
.ft B
int BufferLen(b)
	Buffer *b;
.ft
.fi
.PP
.nf
.ft B
char *BufferCharCat(b, c)
	Buffer *b;
	char c;
.ft
.fi
.PP
.nf
.ft B
int BufferTailCmp(b,s)
	Buffer *b;
	char *s;
.ft
.fi
.PP
.nf
.ft B
char *BufferTailReplace(b,s)
	Buffer *b;
	char *s;
.ft
.fi
.PP
.nf
.ft B
char *BufferTailCut(b,n)
	Buffer *b;
	int n;
.ft
.fi
.PP
.nf
.ft B
char *BufferIncrement()
.ft
.fi
.PP
.nf
.ft B
char *BufferStrRepl()
	Buffer *b1;
	char *s1;
	char *s2;
.ft
.fi
.SS Functions using Buffer
.nf
.ft B
char *BufferGetSubString(s1,o1,l1)
	char *s1;
	int o1;
	int l1;
.ft
.fi
.PP
.nf
.ft B
char *BufferGetIntString(s1)
	char *s1;
.ft
.fi
.SH DESCRIPTION
.PP
These functions allow handling of string of undefined length.
For string manipulation Buffer size becomes length of the biggest treated string.
Input/Output functions use facilities of Realloc Unix function.
.PP
.B Buffer
is an internal type containing hard buffer and management variables.
.PP
.B BufferCreate
is the
.I Buffer
constructor, parameter
.I s
is the initial size of internal buffer, and parameter
.I i
the size of increment. 
When Buffer functions attempt to add some character into Buffer and
when space left is not sufficient, a space reallocation is processed
adding increment to the actual size of Buffer.
.PP
.B BufferString
returns a pointer on the actual string of the given Buffer.
Pay attention: pointer value may be different on different call on the
same buffer (see 
.BR BufferSave ).
.PP
.B BufferFromString
creates a Buffer from a C string. Returns the Buffer created.
.PP
.B BufferFree
frees the content of the Buffer. Should never be used again on the same Buffer (except after another Buffer creator, like 
.B BufferCreate 
or 
.BR BufferFromString ).
.PP
.B BufferReset
reinitializes the content of 
.I b
(it contains an empty string).
.PP 
.B BufferSave
generates a copy of string which is contained into buffer
.IR b .
This function must be used if the string need to be used after next
call on the same buffer. Warning: returned C string must later be freed.
.PP
.B BufferCopy
copies content of Buffer 
.I b2
into
.IR b1 .
Warning: 
.I b1 
must already exist.
.PP
.B BufferClone
creates a new Buffer whose content is a copy of 
.IR b1 .
.PP
.B BufferStrcat
catenates string
.I s
at the end of buffer
.IR b .
This function returns a pointer on actual beginning of buffer string.
Pay attention: pointer value may be different on different call on the
same buffer.
.PP
.B BufferStrncat
catenates 
.I n
first bytes of string
.I s
at the end of buffer
.IR b .
This function returns pointer on actual beginning of buffer string.
Pay attention: pointer value may be different on different call on the
same buffer.
.PP
.B BufferFgets
reads a record, using buffer
.I b
from stream 
.I s.
.B BufferGets 
do the same thing on standard input.
Both fonctions returns a pointer on the read string and NULL
pointer at the end of file.
.PP
.B BufferSize 
gives the actual size of the internal Buffer (depends on size and increment parameters given to 
.BR BufferCreate ).
.PP
.B BufferLen
gives the length of the current string in the Buffer.
.PP
.BR BufferStrcpy ,
.B BufferStrncpy
apply 
.BR string (3C)
unix function to a given buffer. 
Each function returns pointer on the beginning of internal string of buffer.
.PP
.B BufferCat
catenates buffer
.I b2
to buffer
.I b1
and returns the corresponding C string (warning: this string is temporary).
.PP
.B BufferCharCat
catenates char
.I c
to Buffer
.IR b .
This function is macro-implemented and does not return significant
value. 
.PP
.B BufferTailCmp
applies 
.BR strcmp (3C)
function to the tail of 
.I b
given buffer with the string
.IR s .
This function may be used for testing suffix of a given string.
.PP
.B BufferTailCut
cuts the last
.I n
characters of the buffer.
.PP
.B BufferTailReplace
replaces the tail of the given buffer
.I b
by the string
.IR s .
.PP
.B BufferIncrement
increments the size of
.I b1 
by the value given to its constructor.
.PP
.B BufferStrRepl
replaces each occurrence of 
.I s1 
with
.I s2 
in 
.IR b1 .
.PP
.B BufferGetSubString
gets a copy of the C sub-string beginning at offset
.I o1 
with the length 
.I l1 
from the C string
.IR s1.
Warning: the pointer returned may vary with time. It is aimed to be
used before any other Buffer operation.
.PP
.B BufferGetIntString
returns a C string containing the first digit characters of 
.I s1 .
.SH "ERRORS AND RETURN VALUES"
Most of functions returns a pointer on the string which is handed by
Buffer. Only creators (
.BR BufferFromString , 
.BR BufferClone " and " BufferCreate )
return a pointer on the Buffer object.
.PP
If  there  is  no  available  memory, these functions set the
Exception [Buffer - MA] which causes an exit with the implicit action. 
If the Exception (Buffer MA) is inhibited
they return a NULL pointer (see $DILIB/Sample/DilibUnix/Except/ALIRE). 
.SH WARNING
Due to realloc mecanism,
adresses of records can be different on two consecutive calls.
For the same reason, strings made with
.B BufferFgets 
or 
.B BufferGets
must be copied before next call on
same buffer, if they have to be used after -
.B BufferClone 
can be used.
.SH EXAMPLE
.PP
Reading a stream without knowing record size:
.PP
.nf
	#include "Buffer.h"
	main()
	{
		Buffer* Buf;
		char *line;

		Buf = BufferCreate(132, 132);
		while (line=BufferGets(Buf))
			{
			...
			}
	}
.PP
Catenating some strings.
.PP
.nf
    Buffer *b1;
    char *s1;
    b1=BufferFromString("beginning");
    BufferStrcat(b1," middle ");
    s1=BufferSave(BufferStrcat(b1," end"));

.SH AUTHOR
INIST, LORIA : J. DUCLOY, F. PARMENTIER
.SH "SEE ALSO"
.BR malloc (3),
.BR string (3C),
.BR BufferIntro (1).
.PP
Some facilities are a simplified version of Ilib IBuffer module.
