/*   -*- coding: utf-8 -*-  */

/**********************************************************************
*
*  module   : Ccsd
*  comande  : CcsdHal2CorpusStep2
*  fichier  : CcsdHal2CorpusStep2.c
*  Auteur   : Jacques DUCLOY
*  Date     : 2013
*
***********************************************************************/

#include <stdio.h>     /* for printf */
#include <stdlib.h>    /* for exit */
#include <getopt.h>
#include <ctype.h>
#include "TeiHandler.h"
#include "Buffer.h"
#include "Utf8Converter.h"

SxmlNode *docInput;
SxmlNode *corpusRecord;
SxmlNode *teiRoot;

SxmlNode *halFilesUrl;
SxmlNode *halNodeLangue;
char     *halLangue;  /* element langue   */
char     *halL1;
char     *halL2;
SxmlNode *teiKeywords;
char     *halIdArticle;

Buffer *bufName;
Buffer *bufSort;

char *Utf8TestFrEnInKwd (char *text)
{
  char *ret;
  ret ="inc";
  if (strstr(text,"é")) return "fr";
  if (strstr(text,"è")) return "fr";
  if (strstr(text,"y ")) return "en";
  return "inc";
}

char *findSemiColon(char *s1)
{
  char *posSemiColon;
  char *posAmp;
  posSemiColon=strchr (s1, ';');
  if (!posSemiColon)return NULL;
  posAmp=strchr (s1, '&');
  if(!posAmp)return posSemiColon;
  if(posSemiColon < posAmp) return posSemiColon;
  return findSemiColon(posSemiColon+1);
}

SxmlNode *appendTermsUsingSep(SxmlNode *keywords, char *str, int sep)
{
  char *posSep;
  char *s1;
  unsigned char firstChar;
  static Buffer *bufTerm=NULL;
  if (!bufTerm)bufTerm=NewBuffer();
  s1=str;
  while (*s1==' ')s1++;
  if ((posSep=strchr(s1, sep))) 
    {
      if (sep==';')
	{
	  char *posAmp;
	  if ((posAmp=strchr(s1,'&')))
	    {
	      if (posAmp>posSep)BufferStrncpy(bufTerm, s1, posSep-s1);
	      else 
		{
		  posSep=findSemiColon(s1);
		  if (posSep) BufferStrncpy(bufTerm, s1, posSep-s1);
		}
	    }
	  else BufferStrncpy(bufTerm, s1, posSep-s1);
	}
      else BufferStrncpy(bufTerm, s1, posSep-s1);
    }
  else BufferStrcpy(bufTerm, s1);
  while ((BufferTailCmp(bufTerm, " ")==0)) BufferTailCut(bufTerm, 1);
  firstChar=*BufferString(bufTerm);
  if (islower(firstChar)) *BufferString(bufTerm)=toupper(firstChar); 
  else
    {
      if ((firstChar== 0xC3) 
	  && ((unsigned char)(BufferString(bufTerm)[1]) == 0xA9)) 
	BufferString(bufTerm)[1]= 0x89; /* é -> É */
    }
  SxmlAppendChild(keywords, SxmlLeafCreate("term", BufferString(bufTerm)));
  if (posSep) return appendTermsUsingSep(keywords, posSep+1, sep);
  return keywords;
  /* return SxmlAppendChild(keywords, SxmlLeafCreate("term", s1)); */
}

SxmlNode *computeHalKwd (char *kwdType)
{
  SxmlNode *halKeywords;
  SxmlNode *halKwdType;
  char *strKw;
  char *posColon;
  halKeywords=SxmlGetFirstChildByTagName(docInput,kwdType);
  if (!halKeywords)return NULL;
  strKw=SxmlLeafText(halKeywords);
  if (!strKw)return NULL;
  if (strlen(strKw)==0)return NULL; 
  if (!teiKeywords) SxmlAppendChild(Tei_textClass, teiKeywords=SxmlElementCreate("hal:keywords")); 
  SxmlAppendChild(teiKeywords, halKwdType=SxmlElementCreate(kwdType)); 
  SxmlSetAttribute (halKwdType, "xml:lang", Utf8TestFrEnInKwd (strKw));
  if ((posColon=strchr(strKw,';'))) appendTermsUsingSep(halKwdType, strKw,';'); 
  else 
    {
      if ((posColon= strchr(strKw,','))) appendTermsUsingSep(halKwdType, strKw,',');
      else appendTermsUsingSep(halKwdType, strKw,';'); /* to be improved */
    }
  return halKwdType;
}

void computeHalKeywords()
{
  SxmlNode *teiHalKw;
  SxmlNode *nodeTerm;
  char *langKw;
  teiKeywords=NULL;
  teiHalKw=computeHalKwd ("keyword");
  computeHalKwd ("keyword_ml");
  if (teiHalKw)
    {
      langKw=SxmlGetAttribute(teiHalKw,"xml:lang");
      if (strcmp(langKw, "inc")==0) langKw=halL1;
      if (strcmp(langKw, halL1)==0)
	{
	  SxmlReset(teiHalKw);
	  while ((nodeTerm=SxmlNextNode(teiHalKw)))
	    {
	      TeiKeywordsPutTerm("mix", NULL, NULL, langKw, SxmlLeafText(nodeTerm));
	    }
	}
    }
}

void setTeiAuthors()
{
  SxmlNode *authors;
  SxmlNode *author;
  if ((authors=SxmlGetFirstChildByTagName(docInput,"authors")))
    {
      SxmlReset(authors);
      while ((author=SxmlNextNode(authors)))
	{
	  SxmlNode *forename;
	  SxmlNode *surname;
	  SxmlNode *othername;
	  SxmlNode *affiliations;
	  SxmlNode *affiliation;
	  SxmlNode *halAffiliation;
	  SxmlNode *teiAffiliation;
	  SxmlNode *teiAuthor;
	  SxmlNode *teiName;
	  char *sortSurname;
	  
	  forename=SxmlGetFirstChildByTagName(author,"forename");
	  surname=SxmlGetFirstChildByTagName(author,"surname");
	  othername=SxmlGetFirstChildByTagName(author,"othername");
	  affiliations=SxmlGetFirstChildByTagName(author,"affiliations");
	  
	  teiAuthor=SxmlElementCreate("author");
	  SxmlAppendChild(Tei_titleStmt,teiAuthor);
	  
	  BufferReset(bufName);
	  if(SxmlIsLeaf(forename))
	    {
	      BufferStrcpy(bufName, SxmlLeafText(forename));
	      BufferStrcat(bufName, " ");
	      BufferStrcat(bufName, SxmlLeafText(surname));
	    }
	  else 
	    {
	      if(SxmlIsLeaf(surname)) BufferStrcpy(bufName, SxmlLeafText(surname));
	      else continue;
	    }

	  SxmlAppendChild(teiAuthor, teiName=SxmlLeafCreate("name", BufferString(bufName)));
	  if(SxmlIsLeaf(surname))
	    {
	      sortSurname=Utf8NameToWikiSort(SxmlLeafText(surname));	  
	      BufferStrcpy(bufSort, sortSurname );
	    }
	  else BufferReset(bufSort);
	  if(SxmlIsLeaf(forename))
	    {
	      BufferStrcat(bufSort, ", ");
	      BufferStrcat(bufSort, Utf8NameToWikiSort(SxmlLeafText(forename)));
	    }
	  
	  SxmlSetAttribute(teiName, "sortKey", BufferString(bufSort));
	  
	  if (affiliations) 
	    {
	      SxmlReset(affiliations);
	      while ((affiliation=SxmlNextNode(affiliations)))
		{
		  SxmlAppendChild(teiAuthor, teiAffiliation=SxmlElementCreate("affiliation"));
		  
		  halAffiliation=SxmlClone(affiliation);
		  
		  SxmlSetNodeName(halAffiliation,strdup("hal:affiliation"));
		  SxmlAppendChild(teiAffiliation, halAffiliation);
		}
	    }
	  SxmlAppendChild(Tei_analytic,SxmlClone(teiAuthor));
	} /* end while author */
    }  /* end if authors */
}

void computeTitle()
{
  SxmlNode *title;
  if ((title=SxmlGetFirstChildByTagName(docInput,"title")))
    {
      /*      SxmlAppendChild(Tei_titleStmt,SxmlClone(title)); */
      TeiHandlerSetAnalyticStmtTitle(title, halL1);
    }
}

void computeId()
{
  SxmlNode *idArt;
  idArt=SxmlGetFirstChildByTagName(docInput,"id_article");
  if (!idArt)return;
  halIdArticle=SxmlLeafText(idArt);
  TeiSetIdnoRbid("Hal", halIdArticle);
}

void computeLangue()
{
  halL1=NULL;
  halL2=NULL;
  if((halNodeLangue=SxmlGetFirstChildByTagName(docInput, "langue")))
    {
      if((halLangue=SxmlLeafText(halNodeLangue)))
	{
	  if ((strcmp(halLangue, "Français")==0))
	    {
	      halL1="fr";
	      halL2="en";
	      return;
	    }
	  if ((strcmp(halLangue, "Anglais")==0))
	    {
	      halL1="en";
	      halL2="fr";
	      return;
	    }
	  halL1= "not yet";
	}

    }
  halL1="no langue";
}

void computeHalAbstract()
{
  SxmlNode *absNode;
  if((absNode=SxmlGetFirstChildByTagName(docInput, "abstract")))
    TeiHandlerStoreAbstract(absNode, halL1);
}

int main()
{
  bufName=BufferCreate(10,10);
  bufSort=BufferCreate(10,10);
  while ((docInput=SxmlInputNextDocumentElement()))
    {
      if(SxmlNodeHasName(docInput,"document"))
	{
	  SxmlNode *docu;
	  SxmlNode *teiDate;
	  SxmlNode *halDate;

	  corpusRecord=SxmlElementCreate("record");

	  SxmlAppendChild(corpusRecord, teiRoot=TeiHandlerNew());
	  SxmlSetAttribute(teiRoot, "xmlns:hal", "http://www.ccsd.fr");

	  docu=SxmlClone(docInput);
	  SxmlSetNodeName(docu,strdup("hal")); /* strdup pour SxmlFree() */
	  SxmlAppendChild(corpusRecord,docu);
	  SxmlSetAttribute(docu, "api", "V2");

	  computeId();
	  computeLangue();
	  computeTitle();

	  if ((halFilesUrl=SxmlGetFirstChildByTagName(docInput,"files_url")))
	    {
	      SxmlNode *fileUrl;
	      while ((fileUrl=SxmlNextNode(halFilesUrl)))
		{
		  if (SxmlNodeHasName(fileUrl, "file_url"))
		    {
		      TeiAppendIdno("url", SxmlLeafText(fileUrl));
		    }
		}
	    }

	  setTeiAuthors();
	  if ((halDate=SxmlGetFirstChildByTagName(docInput,"datepub")))
	    {
	      char *dateStr;
	      dateStr=SxmlLeafText(halDate);
	      if (dateStr)
		{
		  SxmlAppendChild(Tei_publicationStmt, teiDate=SxmlLeafCreate("date", dateStr));
		  if (strlen(dateStr)>3)SxmlSetAttribute(teiDate,"when", dateStr);
		  else SxmlSetAttribute(teiDate,"when", "????");
		}
	      else
		{
		  SxmlAppendChild(Tei_publicationStmt, teiDate=SxmlLeafCreate("date", "????"));
		  SxmlSetAttribute(teiDate,"when", "????");
		}
	    }
	  else
	    {
	      SxmlAppendChild(Tei_publicationStmt, teiDate=SxmlLeafCreate("date", "????"));
	      SxmlSetAttribute(teiDate,"when", "????");
	    }
	  
	  computeHalKeywords();
	  computeHalAbstract();

	  SxmlPrint(corpusRecord);
	  putchar('\n');
	  SxmlFree(corpusRecord);
	}
    }
  exit (EXIT_SUCCESS);
}
