/*   -*- coding: utf-8 -*-  */

/**********************************************************************
*
*  module   : Ccsd
*  comande  : CcsdHalTei2Corpus
*  fichier  : CcsdHalTei2Corpus.c
*  Auteur   : Jacques DUCLOY
*  Date     : 2014
*
***********************************************************************/

#include <stdio.h>     /* for printf */
#include <stdlib.h>    /* for exit */
#include <getopt.h>
#include "SxmlNode.h"
#include "StrDict.h"
#include "Buffer.h"
#include "TeiHandler.h"
#include "Utf8Converter.h"

SxmlNode *docInput;

SxmlNode *halTitleStmt;
SxmlNode *halEditionStmt;
SxmlNode *halPublicationStmt;
SxmlNode *halProfileDesc;
SxmlNode *halSourceDesc;
SxmlNode *halListBibl;
SxmlNode *halListOrg;

char *halId;

SxmlNode *corpusRecord;
SxmlNode *teiRoot;

StrDict *dictOrg;
Buffer *bufName;
Buffer *bufSort;

SxmlNode *computeAuthor(SxmlNode *itemTitleAuthor)
{
  SxmlNode *teiAuthor;
  SxmlNode *teiName;
  SxmlNode *persName;
  SxmlNode *forename;
  SxmlNode *surname;
  char *surnameStr;
  char *forenameStr;
  persName=SxmlGetFirstChildByTagName(itemTitleAuthor, "persName");
  if (!persName)return NULL;
  forename=SxmlGetFirstChildByTagName(persName,"forename");
  surname=SxmlGetFirstChildByTagName(persName,"surname");
  if(!surname)return NULL;
  if(!(SxmlIsLeaf(surname)))return NULL;
  surnameStr=SxmlLeafText(surname);
  forenameStr=NULL;
  if (forename && SxmlIsLeaf(forename)) forenameStr=SxmlLeafText(forename);
  teiName=TeiAuthorNameFromFirstLastNamesStr(forenameStr, surnameStr);
  if (!teiName) return NULL;
  teiAuthor=SxmlElementCreate("author");
  SxmlAppendChild(teiAuthor, teiName);
  return teiAuthor;
}

void computeTitleStmt()
{
  SxmlNode *titleItem;
  SxmlReset(halTitleStmt);
  while ((titleItem=SxmlNextNode(halTitleStmt)))
    {
      if (SxmlNodeHasName(titleItem, "title"))
	{
	   SxmlAppendChild(Tei_titleStmt,SxmlClone(titleItem));
	   SxmlAppendChild(Tei_analytic, SxmlClone(titleItem));
	   continue;
	}
      if (SxmlNodeHasName(titleItem, "author"))
	{
	  /*
	  SxmlNode *persName;
	  SxmlNode *forename;
	  SxmlNode *surname;
	  SxmlNode *othername;
	  SxmlNode *teiName;
	  char *sortSurname;
	  */
	  SxmlNode *halInputAffiliation;
	  SxmlNode *teiAuthor;


	  teiAuthor=computeAuthor(titleItem);
	  if (!teiAuthor)continue;
	  /*
	  if ((persName=SxmlGetFirstChildByTagName(titleItem, "persName")))
	    {
	      forename=SxmlGetFirstChildByTagName(persName,"forename");
	      surname=SxmlGetFirstChildByTagName(persName,"surname");
	      othername=SxmlGetFirstChildByTagName(persName,"othername");
	      BufferReset(bufName);
	      if(SxmlIsLeaf(forename))
		{
		  BufferStrcpy(bufName, SxmlLeafText(forename));
		  BufferStrcat(bufName, " ");
		  BufferStrcat(bufName, SxmlLeafText(surname));
		}
	      else 
		{
		  if(SxmlIsLeaf(surname)) BufferStrcpy(bufName, SxmlLeafText(surname));
		  else continue;
		}
	      SxmlAppendChild(teiAuthor, teiName=SxmlLeafCreate("name", BufferString(bufName)));
	      if(SxmlIsLeaf(surname))
		{
		  sortSurname=Utf8NameToWikiSort(SxmlLeafText(surname));	  
		  BufferStrcpy(bufSort, sortSurname );
		}
	      else BufferReset(bufSort);
	      if(SxmlIsLeaf(forename))
		{
		  BufferStrcat(bufSort, ", ");
		  BufferStrcat(bufSort, Utf8NameToWikiSort(SxmlLeafText(forename)));
		}
	      
	      SxmlSetAttribute(teiName, "sortKey", BufferString(bufSort));
	    }
	  else continue;
	  */
	  if ((halInputAffiliation=SxmlGetFirstChildByTagName(titleItem, "affiliation")))
	    {
	      char *refAff;
	      SxmlNode *org;
	      SxmlNode *org2;
	      SxmlNode *teiAffiliation;
	      SxmlNode *listRelation;
	      SxmlNode *tutelles;
	      SxmlAppendChild(teiAuthor, teiAffiliation=SxmlElementCreate("affiliation"));
	      refAff=SxmlGetAttribute(halInputAffiliation,"ref");
	      org=(SxmlNode *)StrDictSearch(dictOrg,refAff+1);
	      SxmlAppendChild(teiAffiliation, org2=SxmlClone(org));
	      SxmlSetNodeName(org2,strdup("hal:affiliation")); /* strdup pour SxmlFree() */
	      if ((listRelation=SxmlGetFirstChildByTagName(org2,"listRelation")))
		{
		  SxmlNode *relation;
		  SxmlAppendChild(org2, tutelles=SxmlElementCreate("tutelles"));
		  SxmlReset (listRelation);
		  while ((relation=SxmlNextNode(listRelation)))
		    {
		      SxmlNode *tutelle;
		      char *activeAtt;
		      SxmlNode *orgTutelle;
		      activeAtt=SxmlGetAttribute(relation,"active");
		      orgTutelle=NULL;
		      if (activeAtt) orgTutelle=(SxmlNode *)StrDictSearch(dictOrg,activeAtt+1);
		      if(orgTutelle)
			{
			  SxmlNode *newTutelle;
			  SxmlAppendChild(tutelles, tutelle=SxmlClone(relation));
			  SxmlElementResetEmpty(tutelle);     /* Relation  is empty in HAL V3 */
			  SxmlSetNodeName(tutelle,  strdup("tutelle")); 
			  newTutelle=SxmlClone(orgTutelle);
			  SxmlAppendChild(tutelle,  newTutelle);
			}  /* end orgTutelle */
		    } /* end relation */
		} /* end listRelation */
	    } /* endHalInputAffiliation */
	  SxmlAppendChild(Tei_titleStmt, teiAuthor);
	  SxmlAppendChild(Tei_analytic, SxmlClone(teiAuthor));
	}
    }
}

void storeOrg()
{
  SxmlNode *orgNode;
  while ((orgNode=SxmlNextNode(halListOrg)))
    {
      char *id;
      id=SxmlGetAttribute(orgNode, "xml:id");
      StrDictSet(dictOrg, id, (char*) orgNode);
    }
}

char *computeId()
{
  SxmlNode *idnoHalId;
  SxmlNode *idnoHal;
  idnoHalId=SxmlGetFirstChildTagAtt(halPublicationStmt, "idno", "type", "halId");
  halId=SxmlLeafText(idnoHalId);
  TeiSetIdnoRbid("Hal", halId);
  SxmlReset(halPublicationStmt);
  while ((idnoHal=SxmlNextNode(halPublicationStmt)))
    {
      if(SxmlNodeHasName(idnoHal,"idno"))
	{
	  if (SxmlHasAttribute(idnoHal,"type","halId"))
	    {
	      SxmlAppendChild(Tei_publicationStmt, SxmlClone(idnoHal));
	      continue;
	    }
	  if (SxmlHasAttribute(idnoHal,"type","halUri"))
	    {
	      SxmlAppendChild(Tei_publicationStmt, SxmlClone(idnoHal));
	      TeiAppendIdno("url", SxmlLeafText(idnoHal));
	      continue;
	    }
	}
    }
  return halId;
}

SxmlNode *computeDate()
{
  SxmlNode *halDateCreated;
  SxmlNode *edition;
  char *dateStr;
  SxmlNode *teiDate;
  edition=SxmlGetFirstChildByTagName(halEditionStmt,"edition");
  teiDate=NULL;
  while (edition)
    {
      halDateCreated=SxmlGetFirstChildTagAtt(edition, "date", "type", "whenProduced");
      if(halDateCreated)
	{
	  dateStr=SxmlLeafText(halDateCreated);
	  if (dateStr)
	    {
	      SxmlAppendChild(Tei_publicationStmt, teiDate=SxmlLeafCreate("date", dateStr));
	      if (strlen(dateStr)>3)SxmlSetAttribute(teiDate,"when", dateStr);
	      else SxmlSetAttribute(teiDate,"when", "????");
	      return teiDate;
	    }
	}
      edition=SxmlGetNextSiblingByTagName(edition,"edition");
    }
  SxmlAppendChild(Tei_publicationStmt, teiDate=SxmlLeafCreate("date", "????"));
  SxmlSetAttribute(teiDate,"when", "????");
  return teiDate;
}

SxmlNode *computeAbstract()
{
  SxmlNode *halAbstract;
  if ((halAbstract=SxmlGetFirstChildByTagName(halProfileDesc, "abstract")))
    {
      char *lang;
      SxmlNode *res;
      lang=SxmlGetAttribute(halAbstract,"xml:lang");
      res=TeiHandlerStoreAbstract(halAbstract, lang);
      return res;
    }
  return NULL;
}

SxmlNode *computeKeywords()
{
  SxmlNode *halTextClass;
  SxmlNode *kw;
  kw=NULL;
  if ((halTextClass=SxmlGetFirstChildByTagName(halProfileDesc, "textClass")))
    {
      SxmlNode *kOrClass;
      SxmlReset(halTextClass);
      while ((kOrClass=SxmlNextNode(halTextClass)))
	{
	  if (SxmlNodeHasName(kOrClass, "keywords"))
	    {
	      SxmlNode *halTerm;
	      SxmlReset(kOrClass);
	      while ((halTerm=SxmlNextNode(kOrClass)))
		{
		  if (SxmlNodeHasName(halTerm, "term"))
		    {
		      kw=TeiKeywordsPutTerm("mix", NULL, NULL, SxmlGetAttribute(halTerm, "xml:lang"), SxmlLeafText(halTerm));
		    }
		}
	    }
	}
    }
  return kw;
}

SxmlNode *computeSourceDesc()
{
  SxmlNode *halBiblStruct;
  SxmlNode *halMonogr;
  SxmlNode *halTitleJournal;
  SxmlNode *halISSN;
  SxmlNode *halImprint;
  SxmlNode *halDate;
  SxmlNode *targetIssn;
  SxmlNode *targetImprint;
  SxmlNode *halDoiNode;

  if (!halSourceDesc) return NULL;
  halBiblStruct=SxmlGetFirstChildByTagName(halSourceDesc, "biblStruct");
  if (!halBiblStruct) return NULL;
  halDoiNode=SxmlGetFirstChildTagAtt(halBiblStruct, "idno", "type", "doi");
  if (SxmlIsLeaf(halDoiNode)) TeiHandlerSetDoi(SxmlLeafText(halDoiNode));
  halMonogr=SxmlGetFirstChildByTagName(halBiblStruct, "monogr");
  if(!halMonogr) return NULL;
  halTitleJournal=SxmlGetFirstChildTagAtt(halMonogr, "title", "level","j");
  if (!halTitleJournal) return NULL;
  halISSN=SxmlGetFirstChildTagAtt(halMonogr, "idno", "type","issn");
  if (!halISSN) return NULL;
  halImprint=SxmlGetFirstChildByTagName(halMonogr, "imprint");
  if (!halImprint) return NULL;
  halDate=SxmlGetFirstChildTagAtt(halImprint, "date", "type","datePub");
  if (!halDate) return NULL;
  SxmlAppendChild(Tei_biblStruct, Tei_series=SxmlElementCreate("series"));
  SxmlAppendChild(Tei_series, SxmlClone(halTitleJournal));
  SxmlAppendChild(Tei_series, targetIssn=SxmlClone(halISSN));
  SxmlSetAttribute(targetIssn, "type", "ISSN");
  SxmlAppendChild(Tei_series, targetImprint=SxmlElementCreate("imprint"));
  SxmlAppendChild(targetImprint, SxmlClone(halDate));
  return Tei_series;
}

int main()
{
  dictOrg=NewStrDict();
  bufName=BufferCreate(10,10);
  bufSort=BufferCreate(10,10);
  while ((docInput=SxmlInputNextDocumentElement()))
    {
      SxmlNode *textNode;
      if ((textNode=SxmlGetFirstChildByTagName(docInput, "text")))
	{
	  SxmlNode *bodyNode;
	  SxmlNode *backNode;
	  if ((backNode=SxmlGetFirstChildByTagName(textNode, "back")))
	    {
	      if ((halListOrg=SxmlGetFirstChildByTagName(backNode, "listOrg")))
		{
		  storeOrg();
		}
	    }
	  if ((bodyNode=SxmlGetFirstChildByTagName(textNode, "body")))
	    {
	      if ((halListBibl=SxmlGetFirstChildByTagName(bodyNode, "listBibl")))
		{
		  SxmlNode *bibItem;
		  SxmlNode *docu;
		  while ((bibItem=SxmlNextNode(halListBibl)))
		    { 
		      corpusRecord=SxmlElementCreate("record");
		      SxmlSetAttribute(corpusRecord, "xmlns:hal", "http:www.ccsd.fr");
		      SxmlAppendChild(corpusRecord, teiRoot=TeiHandlerNew());
		      docu=SxmlClone(bibItem);
		      TeiAppendIdno("wicri:source","HAL");
		      halTitleStmt=SxmlGetFirstChildByTagName(bibItem, "titleStmt");
		      halEditionStmt=SxmlGetFirstChildByTagName(bibItem, "editionStmt");
		      halPublicationStmt=SxmlGetFirstChildByTagName(bibItem, "publicationStmt");
		      halProfileDesc=SxmlGetFirstChildByTagName(bibItem,"profileDesc");
		      halSourceDesc=SxmlGetFirstChildByTagName(bibItem,"sourceDesc");
		      computeId();
		      computeTitleStmt();
		      computeSourceDesc();
		      computeDate();
		      computeAbstract();
		      computeKeywords();
		      SxmlAppendChild(corpusRecord,docu);
		      SxmlSetNodeName(docu,strdup("hal")); /* strdup pour SxmlFree() */
		      SxmlSetAttribute(docu, "api", "V3");
		      SxmlPrint(corpusRecord);
		      SxmlFree(corpusRecord);
		      putchar('\n');
		    }
		}
	    }
	}
    }
  exit (EXIT_SUCCESS);
}
