/**********************************************************************
*
*  projet   : DilibPro
*  module   : Except
*  fichier  : Except.c
*  Auteur   : Jacques DUCLOY
*  Date     : Janvier 93
*  $Id: Except.c,v 1.2 2005/06/22 15:01:51 parmentf Exp $
*
***********************************************************************
*
* Copyright (c) 1994 CNRS/CRIN & INRIA Lorraine
* 
***********************************************************************/
#include <stdlib.h>
#include <string.h>
#include <stdlib.h>
#include "Except.h"
#include "StrSearch.h"
#include "Buffer.h"

#define BufferSaveCat3(a,b,c)             \
  (((!BufferBuffer)?BufferInit():0),      \
   BufferStrcpy(BufferBuffer,(a)),        \
   BufferStrcat(BufferBuffer,(b)),        \
   BufferStrcat(BufferBuffer,(c)),        \
   BufferSave(BufferBuffer))

Exception  *ExceptLast;
char       *ExceptionCommand=NULL;

int        ExceptFlagHasBeenSet =0;
static int ExceptFirstCall      =1;
void       (* ExceptActionGlobal)();
void       ExceptInit();

Exception *ExceptFreeList;

StrSearchTable *ExceptTableProc;

int ExceptHasBeenSet()
{
  return ExceptFlagHasBeenSet;
}

void ExceptReset()
{
  ExceptFlagHasBeenSet=0;
  if(ExceptLast)
    {
      if(ExceptFreeList)
	{
	  Exception *tail;
	  Exception *last;
	  tail=ExceptFreeList;
	  do {last=tail;} while((tail=tail->prev));
	  tail->prev=ExceptLast;
	}
      else
	{
	  ExceptFreeList=ExceptLast;
	}
      ExceptLast=NULL;
    }
}

void ExceptGarbage()
{
  if(ExceptFreeList)
    {
      Exception *tail;
      Exception *last;
      tail=ExceptFreeList;
      do {last=tail;free (tail);} while((tail=last->prev));
      tail->prev=ExceptLast;
    }
}

Exception *ExceptionCreate(
     char *mod1,                       /* module        */
     char *cod1,                       /* error code    */
     char *mesB1,                      /* error message */
     char *mesP1,                      /* error message */
     char *mesE1,                      /* error message */
     int  lev1)
{
  Exception *d1;
  if(ExceptFreeList)
    {
      d1=ExceptFreeList;
      ExceptFreeList=ExceptFreeList->prev;
    }
  else
    {
      d1=(Exception *)malloc(sizeof(Exception));
    }

  d1->module   =mod1;
  d1->code     =cod1;
  d1->messBegin=mesB1;
  d1->messParam=mesP1;
  d1->messEnd  =mesE1;
  d1->level=lev1;

  d1->prev=ExceptLast;
  ExceptLast=d1;

  return d1;

}

Exception *ExceptionCreate5(
     char *mod1,                       /* module        */
     char *cod1,                       /* error code    */
     char *mesB1,                      /* error message */
     char *mesP1,                      /* error message */
     char *mesE1,                      /* error message */
     char *m4,
     char *m5,
     int  lev1)
{
  Exception *d1;
  d1=ExceptionCreate(mod1,cod1,mesB1,mesP1,mesE1,lev1);
  return d1;
}

void ExceptImplicitAction(Exception *d1)
{
  if(d1->level)
    {
      if (ExceptionCommand(d1))
	{
	  fprintf(stderr, "\n*** %s (%s): %s : %s %s %s \n",
              ExceptionCommand(d1),
              ExceptionModule(d1), 
              ExceptionCode(d1), 
	      ExceptionMessBegin(d1),
	      ExceptionMessParam(d1),
	      ExceptionMessEnd(d1));
	}
      else
	{
	  fprintf(stderr,"\n*** %s: %s : %s %s %s \n", 
              ExceptionModule(d1), 
              ExceptionCode(d1), 
	      ExceptionMessBegin(d1),
	      ExceptionMessParam(d1),
	      ExceptionMessEnd(d1));
	}
      exit(1);
    }
  else return;
}

void ExceptSetError(
     char *mod1,                       /* module        */
     char *cod1,                       /* error code    */
     char *mesB1,                      /* error message */
     char *mesP1,                      /* error message */
     char *mesE1,                      /* error message */
     int   lev1)
{
  Exception *d1;
  void (*f1)();

  if(ExceptFirstCall)ExceptInit();

  d1=ExceptionCreate(mod1,cod1,mesB1,mesP1,mesE1,lev1);
  
  ExceptFlagHasBeenSet=1;
  if ((f1=(void (*)())StrSearch(ExceptTableProc,BufferSaveCat3(mod1,"/",cod1))))
    {
      f1(d1);
      return;
    };
  if ((f1=(void (*)())StrSearch(ExceptTableProc,mod1)))
    {
      f1(d1);
      return;
    };

  ExceptActionGlobal(d1);
}

void ExceptSetError5(
     char *mod1,                      /* module        */
     char *cod1,                      /* error code    */
     char *mes1,                      /* error message */
     char *mes2,                      /* error message */
     char *mes3,                      /* error message */
     char *mes4,                      /* error message */
     char *mes5,                      /* error message */
     int   lev1)
{
  Exception *d1;
  void (*f1)();
  static Buffer *internalBuffer=NULL;

  if(ExceptFirstCall)ExceptInit();

  /* TODO */
  /* Il faudrait simplifier la structure de Exception pour qu'il n'y
     ait qu'un seul message.  Ainsi, on pourrait y mettre un message
     compose d'autant de sous-chaines qu'on veut. MessBegin n'est
     utilise que dans des messages d'erreurs. */
  d1=ExceptionCreate5(mod1,cod1,mes1,mes2,mes3,mes4,mes5,lev1);
  
  ExceptFlagHasBeenSet=1;
  if(!internalBuffer)internalBuffer=BufferCreate(16,16);
  BufferStrcpy(internalBuffer,mod1);
  BufferStrcat(internalBuffer,"/");
  BufferStrcat(internalBuffer,cod1);
  /* TODEL */
/*   if ((f1=(void (*)())StrSearch(ExceptTableProc,BufferSaveCat3(mod1,"/",cod1)))) */
  if ((f1=(void (*)())StrSearch(ExceptTableProc,BufferSave(internalBuffer))))
    {
      f1(d1);
      return;
    };
  if ((f1=(void (*)())StrSearch(ExceptTableProc,mod1)))
    {
      f1(d1);
      return;
    };

  ExceptActionGlobal(d1);
}

void ExceptOnGlobal(void (*f1)())
{
  ExceptActionGlobal=f1;
}

void ExceptOnModule(char *m1, void (*f1)())
{
  if(ExceptFirstCall)ExceptInit();
 StrSearchPut(ExceptTableProc,m1,f1);
}

void ExceptOnCode(char *m1, char *c1, void (*f1)())
{
  if(ExceptFirstCall)ExceptInit();
  StrSearchPut(ExceptTableProc,BufferSaveCat3(m1,"/",c1),f1);
}

void ExceptResetGlobal()
{
  ExceptActionGlobal=ExceptImplicitAction;
}

void ExceptInit()
{
  ExceptFirstCall=0;
  ExceptActionGlobal=ExceptImplicitAction;
  ExceptLast=NULL;
  ExceptFreeList=NULL;
  ExceptTableProc= StrSearchTableCreate(20,20);
}

void ExceptPrint()
{
  Exception *d1;
  if((d1=ExceptLast))
    do
      {
      fprintf(stderr, "\n*** %s : %s : %s %s %s \n", 
              ExceptionModule(d1), 
              ExceptionCode(d1), 
	      ExceptionMessBegin(d1),
	      ExceptionMessParam(d1),
	      ExceptionMessEnd(d1));
      }
    while((d1=d1->prev));
}

