/*   -*- coding: utf-8 -*-  */

/*
   This programs converts results from a Query on ProdInra (this file must have been formatted by SxmlUnIndent
 */

#include <stdio.h>     /* for printf */
#include <stdlib.h>    /* for exit */
#include <getopt.h>
#include "SxmlNode.h"
#include "TeiHandler.h"

SxmlNode *docInput;
char *yearInraStr;

void computeCreator(SxmlNode *creator)
{
  SxmlNode *authorInra;
  SxmlNode *firstName;
  SxmlNode *lastName;
  SxmlNode *newName;
  SxmlNode *newAuthor;
  SxmlNode *inraAffiliation;
  SxmlNode *externalAffiliation;
  SxmlNode *teiInraAffiliation;
  SxmlNode *teiAffiliation;
  authorInra=SxmlGetFirstChildByTagName(creator,"author");
  if (!authorInra) return;
  lastName=SxmlGetFirstChildByTagName(authorInra,"lastName");
  if (!lastName) return;
  firstName=SxmlGetFirstChildByTagName(authorInra,"firstName");
  if (!firstName)  firstName=SxmlGetFirstChildByTagName(authorInra,"initial");
  newName=TeiAuthorNameFromFirstLastNames(firstName, lastName);
  SxmlAppendChild(Tei_analytic, newAuthor=SxmlElementCreate("author"));
  SxmlAppendChild(newAuthor, newName);
  if ((inraAffiliation=SxmlGetFirstChildByTagName(authorInra,"inraAffiliation")))
    {
      SxmlAppendChild(newAuthor, teiAffiliation=SxmlElementCreate("affiliation"));
      SxmlAppendChild(teiAffiliation, teiInraAffiliation=SxmlClone(inraAffiliation));
      SxmlSetNodeName(teiInraAffiliation, strdup("inra:affiliation"));
      SxmlSetAttribute(teiInraAffiliation, "type", "INRA");
    }
  if ((externalAffiliation=SxmlGetFirstChildByTagName(authorInra,"externalAffiliation")))
    {
      SxmlAppendChild(newAuthor, teiAffiliation=SxmlElementCreate("affiliation"));
      SxmlAppendChild(teiAffiliation, teiInraAffiliation=SxmlClone(externalAffiliation));
      SxmlSetNodeName(teiInraAffiliation, strdup("inra:affiliation"));
      SxmlSetAttribute(teiInraAffiliation, "type", "external");
    }
}

void computeAbstract(SxmlNode *abstract)
{
  char *language;
  language=SxmlGetAttribute(abstract,"language");
  TeiHandlerStoreAbstract(abstract, TeiLangC3ToC2(language));
}

void computeCollection(SxmlNode *collection)
{
  SxmlNode *issnInra;
  SxmlNode *titleInra;
  if ((issnInra=SxmlGetFirstChildByTagName(collection, "issn")))
    {
      TeiHandlerSetIssn(SxmlLeafText(issnInra));
      if ((titleInra=SxmlGetFirstChildByTagName(collection, "title")))
	{
	  SxmlNode *titleTei;
	  titleTei=SxmlLeafCreate("title", SxmlLeafText(titleInra));
	  SxmlAppendChild(Tei_series, titleTei);
	  SxmlSetAttribute(titleTei,"level","j");
	  if (yearInraStr)
	    {
	      SxmlNode *teiImprint;
	      SxmlNode *teiImprintDate;
	      SxmlAppendChild(Tei_series, teiImprint=SxmlElementCreate("imprint"));
	      SxmlAppendChild(teiImprint, teiImprintDate=SxmlLeafCreate("date", yearInraStr )); 
	      SxmlSetAttribute(teiImprintDate, "type", "published");
	      SxmlSetAttribute(teiImprintDate, "when", yearInraStr);
	    }
	}
    }
}

SxmlNode *computeRecord(SxmlNode *record)
{
  SxmlNode *outputRecord;
  SxmlNode *teiRoot;
  SxmlNode *yearInra;
  SxmlNode *titleInra;
  SxmlNode *idInra;
  SxmlNode *doiInra;
  SxmlNode *creatorInra;
  SxmlNode *collectionInra;
  SxmlNode *abstractInra;

  outputRecord=SxmlElementCreate("record");
  SxmlAppendChild(outputRecord, teiRoot=TeiHandlerNew());
  TeiAppendIdno("wicri:source","INRA");
  if ((titleInra=SxmlGetFirstChildByTagName(record,"title")))
    {
      char *language;
      language=SxmlGetAttribute(titleInra,"language");
      TeiHandlerSetAnalyticStmtTitleStr(SxmlLeafText(titleInra), TeiLangC3ToC2(language));
    }
  if ((idInra=SxmlGetFirstChildByTagName(record,"identifier")))
    {
      TeiSetIdnoRbid("INRA", SxmlLeafText(idInra));
      TeiHandlerSetUrl("http://prodinra.inra.fr/record/",SxmlLeafText(idInra));
    }
  if ((yearInra=SxmlGetFirstChildByTagName(record,"year")))
    {
      yearInraStr=SxmlLeafText(yearInra);
      TeiSetDatePublication(yearInraStr);
    }
  creatorInra=SxmlGetFirstChildByTagName(record,"creator");
  while (creatorInra)
    {
      computeCreator(creatorInra);
      creatorInra=SxmlGetNextSiblingByTagName(creatorInra,"creator");
    }
  if ((collectionInra=SxmlGetFirstChildByTagName(record,"collection")))
    {
      computeCollection(collectionInra);
    }
  if ((doiInra=SxmlGetFirstChildByTagName(record,"doi")))
    {
      TeiHandlerSetDoi(SxmlLeafText(doiInra));
    }
  abstractInra=SxmlGetFirstChildByTagName(record,"abstract");
  while (abstractInra)
    {
      computeAbstract(abstractInra);
      abstractInra=SxmlGetNextSiblingByTagName(abstractInra,"abstract");
    }
  return outputRecord;
}

void computeProduit(SxmlNode *produit)
{
  SxmlNode *record;
  SxmlNode *nextRecord;
  SxmlNode *newRecord;

  record=SxmlGetFirstChildByTagName(produit,"record");
  newRecord=computeRecord(record);
  nextRecord=SxmlNextSibling(record);
  SxmlSetNodeName(record,strdup("INRA")); /* strdup pour SxmlFree() */
  SxmlAppendChild(newRecord, SxmlRemoveChild(record));

  while(nextRecord)
    {
      SxmlNode *nextNextRecord;
      nextNextRecord=SxmlNextSibling(nextRecord);
      SxmlAppendChild(record, SxmlRemoveChild(nextRecord));
      nextRecord=nextNextRecord;
    }
  SxmlPrint(newRecord);
  putchar ('\n');
}

int main()
{
  while ((docInput=SxmlInputNextDocumentElement()))
    {
      SxmlNode *produit;
      if (!SxmlNodeHasName(docInput, "ns2:produits"))continue;
      produit=SxmlGetFirstChildByTagName(docInput,"produit");
      while(produit)
	{
	  SxmlNode *nextProduit;
	  nextProduit=SxmlGetNextSiblingByTagName(produit, "produit");
	  computeProduit(produit);
	  produit=nextProduit;
	}
    }
  exit (EXIT_SUCCESS);
}
