/*   -*- coding: utf-8 -*-  */

/**********************************************************************
*
*  module   : Istex
*  fichier  : IstexCleanFullText.c 
*  Auteur   : Jacques DUCLOY
*  Date     : fin 2013
*
***********************************************************************/

#include <stdio.h>     /* for printf */
#include <stdlib.h>    /* for exit */
#include <ctype.h>
#include <getopt.h>
#include "SxPath.h"
#include "Buffer.h"
#include "Istex.h" 

extern char *optarg;
extern int   optind;
int getopt();

Buffer *bufMessClean;
Buffer *bufMessToSee;

int kOption;

/* SxmlNode *cleanMetadata(SxmlNode *metaNode, char *corpusName) */
SxmlNode *IstexCleanMetadataXmlOld(SxmlNode *metaXml, char* corpusName)
{
  /*  SxmlNode *metaXml; */
  SxmlNode *document;
  int flagClean;
  int flagToSee;
  SxmlNode *header;
  SxmlNode *itemArticle;
  SxmlNode * nextItem;

  BufferReset(bufMessClean);
  flagClean=0;
  BufferReset(bufMessToSee);
  flagToSee=0;
  header=NULL;

  /*
  metaXml=SxmlGetFirstChildByTagName(metaNode,"istex:metadataXml");
  if (!metaXml)return NULL;
  */
  SxmlSetAttribute (metaXml,"wicri:clean","meta found");
  if (kOption==1) SxmlSetAttribute (metaXml, "wicri:rec", SxmlInputRecordKey);
  document=SxmlGetFirstChildByTagName(metaXml,"istex:document");
  if (!document)return metaXml;
  SxmlSetAttribute (metaXml,"wicri:clean","meta doc found");
  if (strcmp(corpusName, "wiley")==0)
    {
      SxmlNode *component;

      SxmlSetAttribute (metaXml,"wicri:clean","Wiley doc found");
      component=SxmlGetFirstChildByTagName(document, "component");
      if (!component)
	{
	  SxmlSetAttribute (metaXml,"wicri:toSee","Wiley, no component");
	  return metaXml;
	}
      SxmlSetAttribute (metaXml,"wicri:clean","Wiley component found");
      itemArticle=SxmlFirstChild(component);
      while (itemArticle)             /* Warning : iterator is forbiden as nodes can be deleted */
	{
	  nextItem=SxmlNextSibling(itemArticle);
	  if (SxmlNodeHasName(itemArticle, "header")) header=itemArticle;
	  else if  ((SxmlNodeHasName(itemArticle, "body")))
	    {
	      if(flagClean) BufferStrcat(bufMessClean, "; ");
	      else BufferStrcpy(bufMessClean, "Wiley, elements deleted: ");
	      flagClean=1;
	      BufferStrcat(bufMessClean,SxmlNodeName(itemArticle));
	      SxmlSetAttribute(metaXml, "wicri:clean", BufferString(bufMessClean));
	      SxmlFree(itemArticle);
	    }
	  else 
	    {
	      if(flagToSee) BufferStrcat(bufMessToSee, "; element ");
	      else BufferStrcpy(bufMessToSee, "Element ");
	      flagToSee=1;
	      BufferStrcat(bufMessToSee,SxmlNodeName(itemArticle));
	      SxmlSetAttribute(metaXml, "wicri:toSee", BufferString(bufMessToSee));
	    }
	  itemArticle=nextItem;
	}
    }
  else if (strcmp(corpusName, "springer")==0)
    {
      SxmlNode *publisher;
      SxmlSetAttribute (metaXml,"wicri:clean","springer doc found");
      publisher=SxmlGetFirstChildByTagName(document, "Publisher");
      if (!publisher)
	{
	  SxmlSetAttribute (metaXml,"wicri:toSee","Springer, no publisher");
	  return metaXml;
	}
     SxmlSetAttribute (metaXml,"wicri:clean","Springer, Publisher found");
    }
  else if (strcmp(corpusName, "elsevier")==0)
    {
      SxmlNode *Article;
      SxmlSetAttribute (metaXml,"wicri:clean","Elsevier doc found");
      Article=SxmlGetFirstChildByTagName(document, "converted-article");
      if (!Article)
	{
	  Article=SxmlGetFirstChildByTagName(document, "simple-article");
	  if (!Article)
	    {
	      SxmlSetAttribute (metaXml,"wicri:toSee","Elsevier, no converted or simple article");
	      return metaXml;
	    }
	  SxmlSetAttribute (metaXml,"wicri:clean","Elsevier simple-article found");
	}
      else SxmlSetAttribute (metaXml,"wicri:clean","Elsevier converted-article found");
      itemArticle=SxmlFirstChild(Article);
      while (itemArticle)             /* Warning : iterator is forbiden as nodes can be deleted */
	{
	  nextItem=SxmlNextSibling(itemArticle);
	  if (SxmlNodeHasName(itemArticle, "head")) header=itemArticle;
	  else if (SxmlNodeHasName(itemArticle, "simple-head")) header=itemArticle;
	  else if  (SxmlNodeHasName(itemArticle, "item-info"));
	  else if  ((SxmlNodeHasName(itemArticle, "ce:floats"))
		    ||(SxmlNodeHasName(itemArticle, "body"))
		    ||(SxmlNodeHasName(itemArticle, "tail")))
	    {
	      if(flagClean) BufferStrcat(bufMessClean, "; ");
	      else BufferStrcpy(bufMessClean, "Elsevier, elements deleted: ");
	      flagClean=1;
	      BufferStrcat(bufMessClean,SxmlNodeName(itemArticle));
	      SxmlSetAttribute(metaXml, "wicri:clean", BufferString(bufMessClean));
	      SxmlFree(itemArticle);
	    }
	  else 
	    {
	      if(flagToSee) BufferStrcat(bufMessToSee, "; element ");
	      else BufferStrcpy(bufMessToSee, "Element ");
	      flagToSee=1;
	      BufferStrcat(bufMessToSee,SxmlNodeName(itemArticle));
	      SxmlSetAttribute(metaXml, "wicri:toSee", BufferString(bufMessToSee));
	    }
	    
	  itemArticle=nextItem;
	}
    }
  else if (strcmp(corpusName, "ecco")==0)
    {
      SxmlNode *Book;
      SxmlSetAttribute (metaXml,"wicri:clean","Ecco doc found");
      Book=SxmlGetFirstChildByTagName(document, "book");
      if (!Book)
	{
	  SxmlSetAttribute (metaXml,"wicri:toSee","Ecco, no book");
	  return metaXml;
	}
      SxmlSetAttribute (metaXml,"wicri:clean","Ecco book found");
      itemArticle=SxmlFirstChild(Book);
      while (itemArticle)             /* Warning : iterator is forbiden as nodes can be deleted */
	{
	  nextItem=SxmlNextSibling(itemArticle);
	  if (SxmlNodeHasName(itemArticle, "bookInfo")) header=itemArticle;
	  else if  (SxmlNodeHasName(itemArticle, "citation"));
	  else if  ((SxmlNodeHasName(itemArticle, "text")))
	    {
	      if(flagClean) BufferStrcat(bufMessClean, "; ");
	      else BufferStrcpy(bufMessClean, "Ecco, elements deleted: ");
	      flagClean=1;
	      BufferStrcat(bufMessClean,SxmlNodeName(itemArticle));
	      SxmlSetAttribute(metaXml, "wicri:clean", BufferString(bufMessClean));
	      SxmlFree(itemArticle);
	    }
	  else
	    {
	      if(flagToSee) BufferStrcat(bufMessToSee, "; element ");
	      else BufferStrcpy(bufMessToSee, "Element ");
	      flagToSee=1;
	      BufferStrcat(bufMessToSee,SxmlNodeName(itemArticle));
	      SxmlSetAttribute(metaXml, "wicri:toSee", BufferString(bufMessToSee));
	    }
	  itemArticle=nextItem;
	}
    }
  else if (strcmp(corpusName, "eebo")==0)
    {
      SxmlSetAttribute (metaXml,"wicri:clean","Eebo doc found");
    }
  /*
  else if (strcmp(corpusName, "oup")==0)
    {
      SxmlNode *article;
      SxmlSetAttribute (metaXml,"wicri:clean","oup doc found");
      article=SxmlGetFirstChildByTagName(document, "article");
      if (!article)
	{
	  SxmlSetAttribute (metaXml,"wicri:toSee","Oup, no article");
	  return metaXml;
	}

      itemArticle=SxmlFirstChild(article);
      while (itemArticle)        
	{
	  nextItem=SxmlNextSibling(itemArticle);
	  if (SxmlNodeHasName(itemArticle, "front")) header=itemArticle;
	  else if  (((SxmlNodeHasName(itemArticle, "body")))
		    ||(SxmlNodeHasName(itemArticle, "back")))
	    {
	      if(flagClean) BufferStrcat(bufMessClean, "; ");
	      else BufferStrcpy(bufMessClean, "Oup, elements deleted: ");
	      flagClean=1;
	      BufferStrcat(bufMessClean,SxmlNodeName(itemArticle));
	      SxmlSetAttribute(metaXml, "wicri:clean", BufferString(bufMessClean));
	      SxmlFree(itemArticle);
	    }
	  itemArticle=nextItem;
	}
    }
  */
  else
    {
      IstexCleanMetadataXml(metaXml, corpusName);
    }
  if (!header)
    {
      if(flagToSee) BufferStrcat(bufMessToSee, " ; no header");
      else BufferStrcpy(bufMessToSee, "no header");
      SxmlSetAttribute(metaXml, "wicri:toSee", BufferString(bufMessToSee));
    }
  return metaXml;
}

void IstexCleanMetadataListNode(SxmlNode *metadataListNode, char *corpusName)
{
  SxmlNode *metadataNode;
  SxmlReset(metadataListNode);
  while ((metadataNode=SxmlNextNode(metadataListNode)))
    {
      if(SxmlNodeHasName(metadataNode,"istex:metadataXml")) IstexCleanMetadataXmlOld(metadataNode, corpusName);
      else if (SxmlNodeHasName(metadataNode,"mods")) IstexCleanMetadataMods(metadataNode, corpusName);
    }
}

void proceedDoc(SxmlNode *docInput)
{
  SxmlNode *istexRoot;

  if (kOption)IstexCleanHfdKey=SxmlInputRecordKey;
  if (SxmlNodeHasName(docInput, "istex")) istexRoot=docInput;
  else istexRoot=SxmlGetFirstChildByTagName(docInput, "istex");
  if (istexRoot) 
    {
      char *corpusName;
      SxmlNode *corpusNameNode;
      
      corpusNameNode=SxmlGetFirstChildByTagName(istexRoot,"corpusName");
      /* corpusName=SxmlLeafText(SxmlGetFirstChildByTagName(istexRoot,"corpusName")); */
      if ((corpusNameNode)
	  &&(corpusName=SxmlLeafText(corpusNameNode)))
	{
	  SxmlNode *metaNode;
	  SxmlNode *fullTextNode;
	  SxmlNode *enrichNode;
	  SxmlNode *refBibNode;

	  fullTextNode=SxmlGetFirstChildByTagName(istexRoot, "fulltext");
	  if (fullTextNode)
	    {
	      SxmlNode *fullTextTei;
	      SxmlNode *textNode;
	      fullTextTei=SxmlGetFirstChildByTagName(fullTextNode, "istex:fulltextTEI");
	      if (fullTextTei)
		{
		  textNode=SxmlGetFirstChildByTagName(fullTextTei, "text");
		  if (textNode) SxmlFree(textNode);
		}
	    }

	  refBibNode=SxmlGetFirstChildByTagName(istexRoot, "refBibs");
	  while (refBibNode)
	    {
	      SxmlNode *nextNode;
	      nextNode=SxmlGetNextSiblingByTagName(refBibNode,"refBibs");
	      SxmlFree(refBibNode);
	      refBibNode=nextNode;
	    }

	  metaNode=SxmlGetFirstChildByTagName(istexRoot, "metadata");
	  while (metaNode)
	    {
	      IstexCleanMetadataListNode(metaNode, corpusName);
	      metaNode=SxmlGetNextSiblingByTagName(metaNode,"metadata");
	    }

	  enrichNode=SxmlGetFirstChildByTagName(istexRoot, "enrichments");
	  if (enrichNode) SxmlFree(enrichNode);
	}
    }
}

int main(int argc, char **argv)
{
  SxmlNode *docInput;
  int cOption;

  kOption=0;
  IstexCleanMetadataXmlInit();
  while((cOption=getopt(argc,argv,"kt"))!=EOF)
    {switch (cOption)
	{
	case 'k':
	  kOption=1;
	  break;
	case 't':
	  IstexCleanTrace=1;
	  break;
	}
    }
  bufMessClean=NewBuffer();
  bufMessToSee=NewBuffer();
   while ((docInput=SxmlInputNextDocumentElement()))
     { 
       proceedDoc(docInput);
       SxmlPrint (SxmlInputRecord);
     }
   exit(EXIT_SUCCESS);
}
