/*   -*- coding: utf-8 -*-  */

/*                                     Libcurl interface */

#include "Istex.h"
#include <curl/curl.h>
#include "CharSetBuffer.h"
#include "Except.h"

char   *IstexRootUrl    =NULL;
int     IstexTraceLevel =0;
int     IstexCleanFlag  =0; 

Buffer *IstexBufUrlQuery;
Buffer *IstexBufMetaDoc;

CURL *curl_handle;

struct MemoryStruct {
  char *memory;
  size_t size;
};

int IstexCurlFlagMemory;


static size_t WriteMemoryCallback(void *contents, size_t size, size_t nmemb, void *userp)
{
  size_t realsize = size * nmemb;
  struct MemoryStruct *mem = (struct MemoryStruct *)userp;
 
  mem->memory = realloc(mem->memory, mem->size + realsize + 1);
  if(mem->memory == NULL) {
    /* out of memory! */ 
    printf("not enough memory (realloc returned NULL)\n");
    return 0;
  }
 
  memcpy(&(mem->memory[mem->size]), contents, realsize);
  mem->size += realsize;
  mem->memory[mem->size] = 0;
 
  return realsize;
}
 
CURLcode resCurl;
struct MemoryStruct IstexChunk;

void IstexInit()
{
  if (!IstexRootUrl) IstexRootUrl="https://api.istex.fr";
  IstexBufUrlQuery=NewBuffer();
  IstexBufMetaDoc=NewBuffer();
  IstexCurlFlagMemory=0;
  curl_global_init(CURL_GLOBAL_ALL);
  curl_handle = curl_easy_init();
  curl_easy_setopt(curl_handle, CURLOPT_WRITEFUNCTION, WriteMemoryCallback);
  curl_easy_setopt(curl_handle, CURLOPT_WRITEDATA, (void *)&IstexChunk);

  IstexTraceLevel=0;
}

char *IstexSetIdPass(char *id, char *pass)
{
  static Buffer *bufUser=NULL;
  if (!bufUser) bufUser=NewBuffer();
  BufferStrcpy(bufUser, id);
  BufferStrcat(bufUser, ":");
  BufferStrcat(bufUser, pass);
  curl_easy_setopt(curl_handle, CURLOPT_USERPWD, BufferString(bufUser));
  return BufferString(bufUser);
}

char *IstexSetAuthToken(char *token)
{
  Buffer *bufToken=NULL;
  struct curl_slist *list = NULL;
  if (!bufToken)bufToken=NewBuffer();
  BufferStrcpy(bufToken, "Authorization: Bearer ");
  BufferStrcat(bufToken, token);
  list = curl_slist_append(list, BufferString(bufToken));
  curl_easy_setopt(curl_handle, CURLOPT_HTTPHEADER, list);
  return token;
}

void IstexEnd()
{
   curl_easy_cleanup(curl_handle); 
}

char *IstexGetDocumentByUri(char *uri)
{
  if (IstexCurlFlagMemory==1) free(IstexChunk.memory);
  else IstexCurlFlagMemory=1;

  IstexChunk.memory = malloc(1);  /* will be grown as needed by the realloc above */ 
  IstexChunk.size = 0;    /* no data at this point */ 

  curl_easy_setopt(curl_handle, CURLOPT_URL, uri);

  resCurl = curl_easy_perform(curl_handle);

  if(resCurl != CURLE_OK)
    {
      fprintf(stderr, "curl_easy_perform() failed: %s\n",
	      curl_easy_strerror(resCurl));
      exit(EXIT_FAILURE);
    }
  return IstexChunk.memory;
}

void IstexGetAndPrintFullTextItem(SxmlNode *hitsItem, char *typeFullText)
{
  SxmlNode *fulltextNode;
  if ((fulltextNode=SxmlGetFirstChildByTagName(hitsItem,"fulltext")))
    {
      SxmlNode *fulltextItem;
      SxmlNode *idNode;
      char      *istexId;
      idNode=SxmlGetFirstChildByTagName(hitsItem,"id");
      if (!idNode) return;
      istexId=SxmlLeafText(idNode);
      while ((fulltextItem=SxmlNextNode(fulltextNode)))
	{
	  SxmlNode *typeNode;
	  SxmlNode *uriNode;
	  typeNode=SxmlGetFirstChildByTagName(fulltextItem,"extension");
	  if(!typeNode) typeNode=SxmlGetFirstChildByTagName(fulltextItem,"type");
	  if (!typeNode) continue;
	  uriNode=SxmlGetFirstChildByTagName(fulltextItem,"uri");
	  if (!uriNode)continue;
	  if (strcmp(SxmlLeafText(typeNode), typeFullText)==0)
	    {
	      curl_easy_setopt(curl_handle, CURLOPT_URL, SxmlLeafText(uriNode));
	      free(IstexChunk.memory);
	      IstexChunk.memory = malloc(1);  /* will be grown as needed by the realloc above */ 
	      IstexChunk.size = 0;    /* no data at this point */ 
	      resCurl = curl_easy_perform(curl_handle);
	      printf("%s\n",  IstexChunk.memory);
	    }
	}
    }
}


void IstexGetAndPrintMetadataItem(SxmlNode *hitsItem, int outputMode, char *typeMeta)
{
  SxmlNode *metaNode;
  if ((metaNode=SxmlGetFirstChildByTagName(hitsItem,"metadata")))
    {
      SxmlNode *metaItem;
      SxmlNode *idNode;
      char *istexId;
      idNode=SxmlGetFirstChildByTagName(hitsItem,"id");
      if (!idNode) return;
      istexId=SxmlLeafText(idNode);
      while ((metaItem=SxmlNextNode(metaNode)))
	{
	  SxmlNode *typeNode;
	  SxmlNode *uriNode;
	  typeNode=SxmlGetFirstChildByTagName(metaItem,"extension");
	  if(!typeNode) typeNode=SxmlGetFirstChildByTagName(metaItem,"type");
	  if (!typeNode) continue;
	  uriNode=SxmlGetFirstChildByTagName(metaItem,"uri");
	  if (!uriNode)continue;
	  BufferStrcpy(IstexBufUrlQuery, SxmlLeafText(uriNode));         /* if needed to compute uri in the future */
	  if (outputMode==3)
	    {
	      printf("%s\n", BufferString(IstexBufUrlQuery)); 
	      continue;
	    }
	  if (strcmp(SxmlLeafText(typeNode), typeMeta)==0)
	    {
	      if (strcmp(typeMeta, "mods")==0)
		{
		  SxmlNode *resNode;
		  resNode=IstexGetDocumentModsByUri(BufferString(IstexBufUrlQuery), istexId);
		  if (resNode)
		    {
		      SxmlPrint(resNode);
		      putchar('\n');
		      SxmlFree(resNode);
		    }
		}
	      else
		{
		  char *posRoot;
		  char *posFirst;
		  
		  curl_easy_setopt(curl_handle, CURLOPT_URL, BufferString(IstexBufUrlQuery));
		  free(IstexChunk.memory);
		  IstexChunk.memory = malloc(1);  /* will be grown as needed by the realloc above */ 
		  IstexChunk.size = 0;    /* no data at this point */ 
		  resCurl = curl_easy_perform(curl_handle);
		  posRoot=strstr(IstexChunk.memory, "<?");
		  if (posRoot)posRoot=strstr(IstexChunk.memory, "?>");
		  else posRoot=IstexChunk.memory;
		  posRoot=strstr(posRoot, "<");
		  if (!posRoot) ExceptSetError("IstexCurl","SY", "No xml markup in",IstexChunk.memory,"",2);
		  if (strncmp(posRoot,"<!--", 4)==0)
		    {
		      posRoot=strstr(posRoot, "-->");
		      posRoot=strstr(posRoot, "<");
		    }
		  posFirst=strstr(posRoot+3, ">");
		  BufferStrncpy(IstexBufMetaDoc, IstexChunk.memory, posFirst-IstexChunk.memory);
		  printf ("%s istexId=\"%s\"%s\n", BufferString(IstexBufMetaDoc), istexId, posFirst);
		}
	    }
	}
    }
}

char *IstexJsonFindUrl(SxmlNode *jsonDoc, char* metaOrFull, char *typeToFind)
{
  SxmlNode *hits;
  SxmlNode *istexElem;
  hits=SxmlFirstChild(SxmlGetFirstChildByTagName(jsonDoc,"hits"));
  if (!hits)return NULL;
  SxmlReset (hits);
  while ((istexElem=SxmlNextNode(hits)))
    {
      if (SxmlNodeHasName(istexElem, metaOrFull))
	{
	  SxmlNode *jsItem;
	  while ((jsItem=SxmlNextNode(istexElem)))
	    {
	      SxmlNode *typeNode;
	      char *type;
	      typeNode=SxmlGetFirstChildByTagName(jsItem,"extension");
	      type=SxmlLeafText(typeNode);
	      if (strcmp(type, typeToFind)==0)
		return SxmlLeafText(SxmlGetFirstChildByTagName(jsItem, "uri"));
	    }
	}
    }
  return NULL;
}

char *IstexGetQueryUrl(char *query, char *corpus, int fromInt, int sizeInt, char *output, char *sort)
{
  /* BufferStrcpy(IstexBufUrlQuery, "https://api.istex.fr/document/?q="); */
  BufferStrcpy(IstexBufUrlQuery, IstexRootUrl);
  BufferStrcat(IstexBufUrlQuery,"/document/?q=");
  CharSetUrlBufferCatFromStr(IstexBufUrlQuery, query);
  if (output)
    {
      BufferStrcat(IstexBufUrlQuery, "&output=");
      BufferStrcat(IstexBufUrlQuery, IstexOutputParam(output));
    }
  if (sort)
    {
      BufferStrcat(IstexBufUrlQuery, "&sortBy=");
      BufferStrcat(IstexBufUrlQuery, IstexOutputParam(sort));
    }
  /*
  else
    {
      BufferStrcat(IstexBufUrlQuery, "&sortBy=qualityIndicators.score,title");
    }
  */
  /* else BufferStrcat(IstexBufUrlQuery, "&output=metadata"); */
  BufferStrcat(IstexBufUrlQuery, "&size=");
  BufferIntCat(IstexBufUrlQuery, sizeInt);
  BufferStrcat(IstexBufUrlQuery, "&from=");
  BufferIntCat(IstexBufUrlQuery, fromInt);
  if (corpus)
    {
      BufferStrcat(IstexBufUrlQuery, "&corpus=");
      BufferStrcat(IstexBufUrlQuery, corpus);
    }
  if (IstexTraceLevel > 0)printf ("%s\n", BufferString(IstexBufUrlQuery));
  return BufferString(IstexBufUrlQuery);
}

char *IstexOutputParam (char *param)
{
  if (strlen(param)>1)return param;
  switch(param[0])
    {
    case '*':
      return param;
    case '3':
      return "id,score,corpusName,author,language,originalGenre,qualityIndicators,title,abstract,subject,host,genre,publicationDate,copyrightDate,pmid,doi,serie,enrichments,fulltext,metadata,refBibs";
    case '2':
      return "id,score,corpusName,author,language,originalGenre,qualityIndicators,title,abstract,subject,host,genre,publicationDate,copyrightDate,pmid,doi,serie,enrichments,fulltext,metadata";
    case '1':
      return "id,score,corpusName,author,language,originalGenre,qualityIndicators,title,abstract,subject,host,genre,publicationDate,copyrightDate,pmid,doi,serie";
    default:
      return param;
    }
}

char *IstexGetIdQueryUrl(char *id, char *output)
{
  BufferStrcpy(IstexBufUrlQuery, IstexRootUrl);
  BufferStrcat(IstexBufUrlQuery,"/document/?q=id:");
  BufferStrcat(IstexBufUrlQuery,id);
  if (output)
    {
      BufferStrcat(IstexBufUrlQuery, "&output=");
      BufferStrcat(IstexBufUrlQuery, IstexOutputParam(output));
    }
  if (IstexTraceLevel > 0)printf ("%s\n", BufferString(IstexBufUrlQuery));
  return BufferString(IstexBufUrlQuery);
}

char *IstexGetDoiQueryUrl(char *id, char *output)
{
  static Buffer *bufQuery=NULL;
  SxmlNode *resQ1;
  SxmlNode *hitsNode;
  SxmlNode *jsonItem;
  SxmlNode *idNode;
  if (!bufQuery) bufQuery=NewBuffer();
  BufferStrcpy(bufQuery, "doi:%22");
  BufferStrcat(bufQuery, id);
  BufferStrcat(bufQuery, "%22");
  resQ1=IstexGetQueryResultSxml(BufferString(bufQuery), NULL, 0, 1, NULL, NULL);
  if (!resQ1) return NULL;
  if ( IstexTraceLevel >0) {SxmlPrint(resQ1); putchar('\n');}
  hitsNode=SxmlGetFirstChildByTagName(resQ1,"hits");
  if (!hitsNode)return NULL;
  jsonItem=SxmlGetFirstChildByTagName(hitsNode,"json:item");
  if (!jsonItem)return NULL;
  idNode=SxmlGetFirstChildByTagName(jsonItem,"id");
  if (!idNode) return NULL;


  BufferStrcpy(IstexBufUrlQuery, IstexRootUrl);
  BufferStrcat(IstexBufUrlQuery,"/document/?q=id:");
  BufferStrcat(IstexBufUrlQuery,SxmlLeafText(idNode));
  if (output)
    {
      BufferStrcat(IstexBufUrlQuery, "&output=");
      BufferStrcat(IstexBufUrlQuery, IstexOutputParam(output));
    }
  if (IstexTraceLevel > 0)printf ("%s\n", BufferString(IstexBufUrlQuery));
  return BufferString(IstexBufUrlQuery);
}




char *IstexGetQueryResult(char *query, char *corpus, int fromInt, int sizeInt, char *output, char *sort)
{
  SxmlNode *resultDoc;
  char *queryUrl;

  if (IstexCurlFlagMemory==1) free(IstexChunk.memory);
  else IstexCurlFlagMemory=1;

  IstexChunk.memory = malloc(1);  /* will be grown as needed by the realloc above */ 
  IstexChunk.size = 0;    /* no data at this point */ 

  queryUrl=IstexGetQueryUrl(query, corpus, fromInt, sizeInt, output, NULL);
  if(curl_handle) {
    /* curl_easy_setopt(curl_handle, CURLOPT_URL, BufferString(IstexBufUrlQuery)); */
    curl_easy_setopt(curl_handle, CURLOPT_URL, queryUrl); 
    /*
    curl_easy_setopt(curl_handle, CURLOPT_WRITEFUNCTION, WriteMemoryCallback);
    curl_easy_setopt(curl_handle, CURLOPT_WRITEDATA, (void *)&IstexChunk);
    */
    /* Perform the request, res will get the return code */ 
    resCurl = curl_easy_perform(curl_handle);
    /* Check for errors */ 
    if(resCurl != CURLE_OK)
      {
	fprintf(stderr, "curl_easy_perform() failed: %s\n",
		curl_easy_strerror(resCurl));
	exit(EXIT_FAILURE);
      }
    /* if ((outputMode==1)||(IstexTraceLevel > 0)) */
    if (IstexTraceLevel > 0)
      {
	printf("%lu bytes retrieved\n", (long)IstexChunk.size);
	printf("%s\n", IstexChunk.memory);
      }

    return IstexChunk.memory;

  }
  exit(EXIT_FAILURE);
}


SxmlNode *IstexGetQueryResultSxml(char *query, char *corpus, int fromInt, int sizeInt, char *output, char *sort)
{
  char *resultStr;
  SxmlNode *resultDoc;
  
  resultStr=IstexGetQueryResult(query, corpus, fromInt, sizeInt, output, sort);
  if (resultStr)
    {
      resultDoc=JsonToSxml(resultStr);
      if (IstexTraceLevel > 0)
	{
	  SxmlPrint(resultDoc);
	  putchar ('\n');
	}
      return resultDoc;
    }
  exit(EXIT_FAILURE);
}
