/*   -*- coding: utf-8 -*-  */

/**********************************************************************
*
*  module   : Nlm
*  fichier  : NlmNcbiMerge.c 
*  Auteur   : Jacques DUCLOY
*  Date     : fin 2015
*
***********************************************************************/

#include <stdio.h>     /* for printf */
#include <stdlib.h>    /* for exit */
#include <getopt.h>
#include "SxPath.h"

char     *previousPmid;
SxmlNode *previousDoc;
SxmlNode *pubMedDoc;
SxmlNode *docInput;

SxPathResult *teiPathPublicationStmt;
SxPathResult *teiPathTextClass;

void proceedMerge()
{
  SxmlNode *pmcPublicationStmt;
  SxmlNode *pmcTextClass;
  SxmlNode *pubMedPublicationStmt;
  SxmlNode *pubMedTextClass;
  SxmlNode *pubMedIdnoNode;   /* idno or date */
  SxmlNode *doubleRec;
  SxmlNode *oldPmcTei;
  SxmlNode *pmcTei;
  SxmlNode *pubmedTei;
  SxmlNode *newRecord;
  SxmlNode *doublePmc;
  SxmlNode *doublePubmed;
  
  newRecord=SxmlElementCreate("record");
  pmcTei=SxmlGetFirstChildByTagName(previousDoc, "TEI");
  SxmlRemoveChild(pmcTei);
  SxmlAppendChild(newRecord, pmcTei);
  SxmlAppendChild(newRecord, doubleRec=SxmlElementCreate("double"));
  SxmlSetAttribute(doubleRec, "pmid", previousPmid);
  SxmlAppendChild(doubleRec, doublePmc=SxmlElementCreate("pmc"));
  SxmlAppendChild(doublePmc, SxmlClone(pmcTei));
  SxmlAppendChild(doubleRec, doublePubmed=SxmlElementCreate("pubmed"));
  pubMedDoc=SxmlRemoveChild(docInput);
  pubmedTei=SxmlGetFirstChildByTagName(pubMedDoc, "TEI");
  SxmlRemoveChild(pubmedTei);
  SxmlFree(pubMedDoc);
  SxmlAppendChild(doublePubmed, pubmedTei);
  /*
  SxmlReplaceName(pubMedDoc, strdup("pubMedRecord"));
  SxmlAppendChild(previousDoc, pubMedDoc);
  pmcPublicationStmt=SxPathFirstResultNode(teiPathPublicationStmt, previousDoc);
  pubMedPublicationStmt=SxPathFirstResultNode(teiPathPublicationStmt, pubMedDoc);
  */
  pmcPublicationStmt=SxPathFirstResultNode(teiPathPublicationStmt, newRecord);
  pubMedPublicationStmt=SxPathFirstResultNode(teiPathPublicationStmt, doublePubmed);
  SxmlReset (pubMedPublicationStmt);
  while ((pubMedIdnoNode=SxmlNextNode(pubMedPublicationStmt)))
    {
      if (SxmlNodeHasName(pubMedIdnoNode, "idno"))
	{
	  char *type;
	  type=SxmlGetAttribute(pubMedIdnoNode, "type");
	  if (strncmp(type,"wicri:",6)==0) SxmlAppendChild(pmcPublicationStmt, SxmlClone(pubMedIdnoNode));
	  if (strcmp(type,"RBID")==0) SxmlAppendChild(pmcPublicationStmt, SxmlClone(pubMedIdnoNode));
	}
    }
  /*
  pmcTextClass=SxPathFirstResultNode(teiPathTextClass, previousDoc);
  pubMedTextClass=SxPathFirstResultNode(teiPathTextClass, pubMedDoc);
  */
  pmcTextClass=SxPathFirstResultNode(teiPathTextClass, newRecord);
  pubMedTextClass=SxPathFirstResultNode(teiPathTextClass, doublePubmed);
  if(pubMedTextClass)
    {
      SxmlNode *pubMedKeywords;
      SxmlReset(pubMedTextClass);
      while((pubMedKeywords=SxmlNextNode(pubMedTextClass)))
	{
	  SxmlAppendChild(pmcTextClass, SxmlClone(pubMedKeywords));
	}
    }
  SxmlPrint(newRecord);
  putchar('\n');
  SxmlFree (previousDoc);
  previousDoc=NULL;
}

void purge()
{
  if (previousDoc)
    {
      SxmlPrint (previousDoc);
      putchar('\n');
      SxmlFree (previousDoc);
      previousDoc=NULL;
    }
  if (previousPmid)
    {
      free(previousPmid);
      previousPmid=NULL;
    }
}

int main()
{

  teiPathPublicationStmt=SxPathFirstCompile("TEI/teiHeader/fileDesc/publicationStmt");
  teiPathTextClass=SxPathFirstCompile("TEI/teiHeader/profileDesc/textClass");

  previousDoc=NULL;
  previousPmid=NULL;
  while ((docInput=SxmlInputNextDocumentElement()))
     { 
       char *inputPmid;
       char *inputType;
       inputPmid=SxmlNodeValue(SxmlFirstChild(SxmlFirstChild(SxmlInputRecord)));
       inputType=SxmlNodeValue(SxmlFirstChild(SxmlNextSibling(SxmlFirstChild(SxmlInputRecord))));
       if (strcmp(inputPmid, "NONE")==0)
	 {
	   purge();
	   SxmlPrint (docInput);
	   putchar('\n');
	   continue;
	 }
       if (strcmp(inputType, "Pmc")==0)
	 {
	   purge();
	   previousDoc=SxmlRemoveChild(docInput);
	   previousPmid=strdup(inputPmid);
	   continue;
	 }
       if (strcmp(inputType, "PubMed")==0)
	 {
	   if (previousPmid)
	     {
	       if (strcmp(previousPmid, inputPmid)==0)
		 {                                        /* doublon */
		   proceedMerge();
		   purge();
		   continue;
		 }
	       else
		 {
		   purge();
		   SxmlPrint (docInput);
		   putchar('\n');
		   continue;
		 }
	     }
	   else
	     {
	       SxmlPrint (docInput);
	       putchar('\n');
	       continue;
	     }
	 }
     }
  purge();
  exit(EXIT_SUCCESS);
}
