.TH StrSearch 3 "7 February 2003" 
.SH NAME
StrSearch, 
.SH SYNOPSIS
.PP
.nf
\#include "StrSearch.h"
.fi
.PP
.nf
.ft B
StrSearchTable\(** StrSearchTableCreate(n,i)
	int n;
	int i;
.ft
.fi
.PP
.nf
.ft B
void StrSearchTableFree(t)
	StrSearchTable\(** t;
.ft
.fi
.PP
.nf
.ft B
void StrSearchTableFreeStr(t)
	StrSearchTable\(** t;
.ft
.fi
.PP
.nf
.ft B
void StrSearchTableFreeKey(t)
	StrSearchTable\(** t;
.ft
.fi
.PP
.nf
.ft B
void StrSearchTableFreeValue(t)
	StrSearchTable\(** t;
.ft
.fi
.PP
.nf
.ft B
StrSearchTable \(**StrSearchAdd(t,k,v)
     StrSearchTable \(**t;
     char \(**k;
     char \(**v;
.ft
.fi
.PP
.nf
.ft B
StrSearchTable \(**StrSearchPut(t,k,v)
     StrSearchTable \(**t;
     char \(**k;
     char \(**v;
.ft
.fi
.PP
.nf
.ft B
char *StrSearch(t,k)
     StrSearchTable *t;
     char *k;
.ft
.fi
.PP
.nf
.ft B
StrSearchTable *StrSearchGetTable(stream,size)
     char *stream;
     int size;
.ft
.fi
.PP
.nf
.ft B
int StrSearchTableToFile(t,s)
     StrSearchTable* t;	
     char *s;
.ft
.fi
.PP
.nf
.ft B
StrSearchTable *StrSearchGetTableFromFile(file,size)
     FILE* file;	
     int size;
.ft
.fi
.PP
.nf
.ft B
StrSearchTable *StrSearchAddTable(stream, table)
     StrSearchTable *table;
     char *stream;
.ft
.fi
.PP
.nf
.ft B
StrSearchTable *StrSearchGetTableC(stream,size)
     char *stream;
     int size;
.ft
.fi
.PP
.nf
.ft B
void StrSearchTableReset(t)
     StrSearchTable* t;
.ft
.fi
.SS Iterator on sorted table
.nf
.ft B
void StrSearchIteratorReset(t)
     StrSearchTable* t;
.ft
.fi
.PP
.nf
.ft B
void StrSearchIteratorReverseReset(t)
     StrSearchTable* t;
.ft
.fi
.PP
.nf
.ft B
char *StrSearchNext(t)
     StrSearchTable* t;
.ft
.fi
.PP
.nf
.ft B
char *StrSearchPrevious(t)
     StrSearchTable* t;
.ft
.fi
.PP
.nf
.ft B
char *StrSearchKey(t)
     StrSearchTable* t;
.ft
.fi
.PP
.nf
.ft B
char *StrSearchValue(t)
     StrSearchTable* t;
.ft
.fi
.PP
.nf
.ft B
.SS approximative searching
.ft
.fi
.PP
.nf
.ft B
char *StrSearchKeyLessEqual(t,s)
     StrSearchTable *t;
     char *s;
.ft
.fi
.SH DESCRIPTION
.PP
.B StrSearch 
is an adaptation of 
.B bsearch(3)
and
.B qsort(3)
to search into string tables.
This package uses an object
.B StrSearchTable
as gateway to unix bsearch functions.
.PP
.B StrSearchTableCreate
creates a StrSearchTable, which is supposed to contain approximatively
.I n
elements.
If 
.I n
is too short, more allocation will automatically be done by increment of
.I i
elements.
.PP
.B StrSearchTableFree
closes table 
.I t
and frees associated structures (only pointers are freed);
.PP
.B StrSearchTableFreeStr
closes table
.IR t ,
frees associated structures and frees all string (keys and values) which are
supposed to be of (char *)type;
.PP
.B StrSearchTableFreeKey
closes table
.IR t ,
frees associated structure and only frees all string keys which are
supposed to be of (char *)type;
.PP
.B StrSearchTableFreeValue
closes table
.IR t ,
frees associated structure and only frees all string values which are
supposed to be of (char *)type;
.PP
.B StrSearchAdd
adds a datum in the table
.IR t , 
with key
.I k
and associated value
.IR v .
One could use any kind of pointer as value, by casting it.
.PP
.B StrSearchPut
adds a new element in the table, or if the key
.I k
exists in the table
.I t
modifies value of corresponding datum by
.IR s .
.PP 
.B StrSearch
searches in table
.I t
value corresponding to key
.I k
and returns adress of value.
This function returns NULL if key is absent. 
This function sets 
.I StrSearchLastKey 
to the key found (NULL if key is absent).
.B bsearch(3)
needs ordered keys, and 
.B StrSearch
will automatically call 
.B qsort(3)
only if required.
.PP
.B StrSearchGetTable
and
.B StrSearchGetTableC
can be used to create a table by reading a file
.I stream
containing approximatively
.I size
lines.
Each line contains key and value separated by tab character.
Values which are read by
.B StrSearchGetTableC
are transcoded using 
.BR CharSetCtoA(3) .
These functions return a pointer on the table or a NULL pointer
if the given stream does not exist.
.PP
.B StrSearchTableToFile
saves the table
.I t
into a formatted text file
.IR f :
one line per record. Those records can be constituted from only
a key or from a pair key-value (the value is separated from the
key by a tabulation).
.PP
.B StrSearchGetTableFromFile
reads a table (which initial 
.I size
is given) from the text
.IR file .
Its format have to be the same as the one produced by
.IR StrSearchTableToFile .
.PP
.B StrSearchAddTable
adds new items coming from the given stream
.I stream
on a previous declared table
.IR table .
Key and value should be on a single line, separated by one
tabulation. Otherwise, only one key per line is added.
.PP
.B StrSearchTableReset
can be used to introduce a new table (new keys and values) into
the area of an existing table
.IR t .
.BR Warning :
does not free any key nor datum.
.PP
.B StrSearchNext
can be used to explore the sorted table. This function returns key of next
datum and NULL at the end of the table.
.PP
.B StrSearchPrevious
can be used to explore the sorted table in reverse order. This function returns key of next
datum and NULL at the end of the table.
.PP
.B StrSearchIteratorReset
must be used before a new sequence of StrSearchNext(). Reinitializes
the Iterator value.
.PP
.B StrSearchIteratorReverseReset
must be used before a new sequence of
StrSearchPrevious(). Reinitializes the Iterator value.
.PP
.B StrSearchValue
and 
.B StrSearchKey
return respectively current value (key) of current datum 
during iterating process.
.PP
.B StrSearchKeyLessEqual
allows retrieval of datum whose key part is equal to a given string
.I s
or the nearest less element.
This function returns key part of datum (or NULL if no
datum can be found).
.B StrSeachValue 
can be used for getting value part.
It's also possible to start StrSearchNext suite of call from selected point.
.SH EXAMPLE
.PP
#include "StrSearch.h"

main()
{
 StrSearchTable *table;
 table = StrSearchTableCreate(10,5);

 StrSearchAdd(table,"arbre","tree");
 StrSearchAdd(table,"maison","home");
 printf("%s\\n", StrSearch(table,"arbre"));    /* prints tree */
 printf("%s\\n", StrSearch(table,"route"));    /* prints (null) */

 StrSearchAdd(table,"chien","dog");
 printf("%s\\n", StrSearch(table,"chien"));   /* sorts table and prints dog */
 ...
}
.SH WARNING
.SS memory allocation
datum are not copied during
.B StrSearchAdd
or
.B StrSearchGetTable
operations.
So they are not freed with
.B StrSearchTableReset
or
.B StrSearchTableFree
.SH "SEE ALSO"
.BR bsearch (3),
.BR qsort (3),
.BR CharSetCtoA (3),
.BR XmlSearchTable (3).
.SH AUTHOR
CNRS
J. DUCLOY, F. PARMENTIER
