/*   -*- coding: utf-8 -*-  */
/***********************************************************************
*
*               Module   : Tei
*               Fichier  : TeiKeywords.c
*               Auteur   : Ducloy
*               Date     : Février 2013
*
************************************************************************
*
* Copyleft 2013 Jacques Ducloy 
* 
************************************************************************/
#include <stdio.h>     /* for printf */
#include <stdlib.h>    /* for exit */
#include <getopt.h>
#include "TeiHandler.h"

SxmlNode *TeiKewordsInsertOrCreateFromScheme (SxmlNode *node, char *scheme, char *type, char*qualifier, char *lang);

SxmlNode *TeiKeywordsFirst()
{
  if (!Tei_textClass) return NULL;
  return SxmlGetFirstChildByTagName(Tei_textClass, "keywords");
}

SxmlNode *TeiKeywordsFirstTerm()
{
  SxmlNode *kw;
  if ((kw=TeiKeywordsFirst()))
    {
       return SxmlGetFirstChildByTagName(kw, "term");
    }
  return NULL;
}

SxmlNode *TeiKeywordsNextFromTerm(SxmlNode *t1)
{
  SxmlNode *t2;
  SxmlNode *k1;
  SxmlNode *k2;
  if ((t2=SxmlGetNextSiblingByTagName(t1,"term"))) return t2;
  k1=SxmlParent(t1);
  if ((k2=SxmlGetNextSiblingByTagName(k1,"keywords"))) return SxmlGetFirstChildByTagName(k2, "term");
  return NULL;
}

SxmlNode *TeiKeywordsGetFirstTermByValue(char *v1)
{
  SxmlNode *t1;
  t1=TeiKeywordsFirstTerm();
  while (t1)
    {
      if (strcmp (SxmlLeafText(t1), v1)==0) return t1;
      t1=TeiKeywordsNextFromTerm(t1);
    }
  return NULL;
}

SxmlNode *TeiFindKeywords(char *scheme, char *type, char*qualifier, char *lang)
{
  SxmlNode *kwNode;
  if (!Tei_textClass) return NULL;
  SxmlReset (Tei_textClass);
  while ((kwNode=SxmlNextNode(Tei_textClass)))
    {
      if (scheme)
	{
	  char *schemeAtt;
	  schemeAtt= SxmlGetAttribute(kwNode,"scheme");
	  if (!schemeAtt)continue;
	  if (strcmp(scheme, schemeAtt)!=0)continue;
	}
      if (type)
	{
	  char *typeAtt;
	  typeAtt= SxmlGetAttribute(kwNode,"type");
	  if (!typeAtt)continue;
	  if (strcmp(type, typeAtt)!=0)continue;
	}
      if (qualifier)
	{
	  char *qualifierAtt;
	  qualifierAtt= SxmlGetAttribute(kwNode,"qualifier");
	  if (!qualifierAtt)continue;
	  if (strcmp(qualifier, qualifierAtt)!=0)continue;
	}
      if (lang)
	{
	  char *langAtt;
	  langAtt= SxmlGetAttribute(kwNode,"xml:lang");
	  if (!langAtt)continue;
	  if (strcmp(lang, langAtt)!=0)continue;
	}
      return kwNode;
    }
  return NULL;
}

SxmlNode *TeiCreateKeywords(char *scheme, char *type, char*qualifier, char *lang)
{
  SxmlNode *outputKW;
  outputKW=SxmlElementCreate("keywords");
  if (scheme)SxmlSetAttribute(outputKW,"scheme", scheme);
  if (type)SxmlSetAttribute(outputKW,"type", type);
  if (qualifier)SxmlSetAttribute(outputKW,"qualifier", qualifier);
  if (lang)SxmlSetAttribute(outputKW,"xml:lang", lang);
  return outputKW;
}

SxmlNode *TeiStoreKeywords(char *scheme, char *type, char*qualifier, char *lang)
{
  SxmlNode *kwNode;
  if (!Tei_textClass) 
    {
      SxmlAppendChild(Tei_profileDesc, Tei_textClass=SxmlElementCreate("textClass"));
      kwNode=TeiCreateKeywords(scheme,type, qualifier, lang);
      SxmlAppendChild( Tei_textClass, kwNode);
      return kwNode;
    }
  if (!SxmlFirstChild(Tei_textClass))
    {
      kwNode=TeiCreateKeywords(scheme,type, qualifier, lang);
      SxmlAppendChild( Tei_textClass, kwNode);
      return kwNode;
    }
  return TeiKewordsInsertOrCreateFromScheme (SxmlFirstChild(Tei_textClass),scheme, type, qualifier, lang);
}

SxmlNode *TeiKewordsInsertOrCreateFromLang (SxmlNode *node, char *scheme, char *type, char*qualifier, char *lang)
{
  SxmlNode *kwNode;
  SxmlNode *newNode;
  char *nodeLang;
  int cmp;
  nodeLang=SxmlGetAttribute(node,"xml:lang");
  if (!nodeLang)
    {
      if (!lang) return node;
      kwNode=TeiCreateKeywords(scheme,type, qualifier, lang);
      SxmlInsertBefore (node, kwNode);
      return kwNode;
    }
  cmp=strcmp(lang, nodeLang);
  if (cmp==0) return node;
  if (cmp >0) 
    {
      if ((newNode=SxmlNextSibling(node))) return TeiKewordsInsertOrCreateFromLang(newNode, scheme, type, qualifier, lang);
      SxmlInsertAfter (node, kwNode=TeiCreateKeywords(scheme,type, qualifier, lang));
       return kwNode;
    }
  kwNode=TeiCreateKeywords(scheme,type, qualifier, lang);
  SxmlInsertBefore (node, kwNode);
  return kwNode;
}

SxmlNode *TeiKewordsInsertOrCreateFromQualifier (SxmlNode *node, char *scheme, char *type, char*qualifier, char *lang)
{
  SxmlNode *kwNode;
  SxmlNode *newNode;
  char *nodeQualifier;
  int cmp;
  nodeQualifier=SxmlGetAttribute(node,"qualifier");
  if (!nodeQualifier)
    {
      if (!qualifier) return TeiKewordsInsertOrCreateFromLang (node, scheme, type, qualifier, lang);
      kwNode=TeiCreateKeywords(scheme,type, qualifier, lang);
      SxmlInsertBefore (node, kwNode);
      return kwNode;
    }
  if (!qualifier)
    {
      if ((newNode=SxmlNextSibling(node))) return TeiKewordsInsertOrCreateFromQualifier(newNode, scheme, type, qualifier, lang);
      SxmlInsertAfter (node, kwNode=TeiCreateKeywords(scheme,type, qualifier, lang));
      return kwNode;
    }
  cmp=strcmp(qualifier, nodeQualifier);
  if (cmp==0) return TeiKewordsInsertOrCreateFromLang (node, scheme, type, qualifier, lang);
  if (cmp >0) 
    {
      if ((newNode=SxmlNextSibling(node))) return TeiKewordsInsertOrCreateFromQualifier(newNode, scheme, type, qualifier, lang);
      SxmlInsertAfter (node, kwNode=TeiCreateKeywords(scheme,type, qualifier, lang));
       return kwNode;
    }
  kwNode=TeiCreateKeywords(scheme,type, qualifier, lang);
  SxmlInsertBefore (node, kwNode);
  return kwNode;
}

SxmlNode *TeiKewordsInsertOrCreateFromType (SxmlNode *node, char *scheme, char *type, char*qualifier, char *lang)
{
  SxmlNode *kwNode;
  SxmlNode *newNode;
  char *nodeType;
  int cmp;
  nodeType=SxmlGetAttribute(node,"type");
  if (!nodeType)
    {
      if (!type) return TeiKewordsInsertOrCreateFromQualifier (node, scheme, type, qualifier, lang);
      kwNode=TeiCreateKeywords(scheme,type, qualifier, lang);
      SxmlInsertBefore (node, kwNode);
      return kwNode;
    }
  if (!type) 
    {
      if ((newNode=SxmlNextSibling(node))) return TeiKewordsInsertOrCreateFromType(newNode, scheme, type, qualifier, lang);
      SxmlInsertAfter (node, kwNode=TeiCreateKeywords(scheme,type, qualifier, lang));
      return kwNode;
    }
  cmp=strcmp(type, nodeType);
  if (cmp==0) return TeiKewordsInsertOrCreateFromQualifier (node, scheme, type, qualifier, lang);
  if (cmp >0) 
    {
      if ((newNode=SxmlNextSibling(node))) return TeiKewordsInsertOrCreateFromType(newNode, scheme, type, qualifier, lang);
      SxmlInsertAfter (node, kwNode=TeiCreateKeywords(scheme,type, qualifier, lang));
       return kwNode;
    }
  kwNode=TeiCreateKeywords(scheme,type, qualifier, lang);
  SxmlInsertBefore (node, kwNode);
  return kwNode;
}

SxmlNode *TeiKewordsInsertOrCreateFromScheme (SxmlNode *node, char *scheme, char *type, char*qualifier, char *lang)
{
  SxmlNode *kwNode;
  SxmlNode *newNode;
  char *nodeScheme;
  int cmp;
  nodeScheme=SxmlGetAttribute(node,"scheme");
  if (!nodeScheme)
    {
      if (!scheme) return TeiKewordsInsertOrCreateFromType (node, scheme, type, qualifier, lang);
      kwNode=TeiCreateKeywords(scheme,type, qualifier, lang);
      SxmlInsertBefore (node, kwNode);
      return kwNode;
    }
  cmp=strcmp(scheme, nodeScheme);
  if (cmp==0) return TeiKewordsInsertOrCreateFromType (node, scheme, type, qualifier, lang);
  if (cmp >0) 
    {
      if ((newNode=SxmlNextSibling(node))) return TeiKewordsInsertOrCreateFromScheme(newNode, scheme, type, qualifier, lang);
      SxmlAppendChild (Tei_textClass, kwNode=TeiCreateKeywords(scheme,type, qualifier, lang));
       return kwNode;
    }
  kwNode=TeiCreateKeywords(scheme,type, qualifier, lang);
  SxmlInsertBefore (node, kwNode);
  return kwNode;
}

SxmlNode * TeiKewordsStoreOrInsertTerm (SxmlNode *inTermNode, char *term)
{
  SxmlNode *newTermNode;
  SxmlNode *nextTermNode;
  char *inTerm;
  int cmp;
  inTerm=SxmlLeafText(inTermNode);
  cmp=strcmp(term, inTerm);
  if (cmp==0) return inTermNode;
  if (cmp < 0)
    {
      newTermNode=SxmlLeafCreate("term",term);
      SxmlInsertBefore(inTermNode, newTermNode);
      return newTermNode;
    }
  if ((nextTermNode=SxmlNextSibling(inTermNode))) return TeiKewordsStoreOrInsertTerm (nextTermNode, term);
  newTermNode=SxmlLeafCreate("term",term);
  SxmlInsertAfter(inTermNode, newTermNode);
  return newTermNode;
}

SxmlNode *TeiKeywordsStoreTerm (SxmlNode *kw, char *term)
{
  SxmlNode *termNode;
  SxmlNode *fTerm;
  if (!(fTerm=SxmlFirstChild(kw)))
    {
      termNode=SxmlLeafCreate("term",term);
      SxmlAppendChild(kw, termNode);
      return termNode;
    }
  return TeiKewordsStoreOrInsertTerm (fTerm, term);
}

SxmlNode *TeiKeywordsPutTerm(char *scheme, char *type, char*qualifier, char *lang, char *term)
{
  SxmlNode *kwNode;
  SxmlNode *termNode;
  kwNode=TeiStoreKeywords(scheme, type, qualifier, lang);
  termNode=TeiKeywordsStoreTerm (kwNode, term);
  return termNode;
}

SxmlNode *TeiKeywordsPutTeiKeyword(SxmlNode *kw)
{
  char *term;
  SxmlNode *termNode;
  char *type;
  char *scheme;
  char *qualifier;
  char *lang;
  SxmlNode *nKw;

  nKw=NULL;
  scheme=SxmlGetAttribute(kw, "scheme");
  type=SxmlGetAttribute(kw, "type");
  qualifier=SxmlGetAttribute(kw, "qualifier");
  lang=SxmlGetAttribute(kw, "xml:lang");
  SxmlReset(kw);
  while ((termNode=SxmlNextNode(kw)))
    {
      term=SxmlLeafText(termNode);
      nKw=TeiKeywordsPutTerm(scheme, type, qualifier, lang, term);
    }
  return nKw;
}
