/*   -*- coding: utf-8 -*-  */
/***********************************************************************
*
*               Module   : Tei
*               Fichier  : TeiKeywordsFromTable.c
*               Auteur   : Ducloy
*               Date     : Février 2013
*
************************************************************************
*
* Copyleft 2013 Jacques Ducloy 
* 
************************************************************************/
#include <stdio.h>     /* for printf */
#include <stdlib.h>    /* for exit */
#include <getopt.h>
#include "TeiHandler.h"
#include "StrSearch.h"

SxmlNode *docInput;
StrSearchTable *matchingTable;

    int getopt();
    extern char *optarg;

char *inputLang;
char *inputScheme;
char *inputType;

char *outputLang;
char *outputScheme;
char *outputType;


int main (int argc, char **argv) {
  int cOption;
  inputLang=NULL;
  inputScheme=NULL;
  inputType=NULL;
  outputLang=NULL;
  outputScheme=NULL;
  outputType=NULL;


  matchingTable=NULL;
  while((cOption=getopt(argc,argv,"s:l:m:t:S:L:T:"))!=EOF)
    
    {switch (cOption)
        {
	case 'm' :
	  matchingTable=StrSearchGetTable(optarg, 100);
	  break;
	case 'l':
	  inputLang=optarg;
	  break;
	case 's':
	  inputScheme=optarg;
	  break;
	case 't':
	  inputType=optarg;
	  break;
	case 'L':
	  outputLang=optarg;
	  break;
	case 'S':
	  outputScheme=optarg;
	  break;
	case 'T':
	  outputType=optarg;
	  break;
	}
    }
  if (! matchingTable) 
    {
      perror("******** -m (matching table is mandatory)");
      exit (1);
    }
  while ((docInput=SxmlInputNextDocumentElement()))
     {   
       SxmlNode *inputKW;
       SxmlNode *outputKW;
       SxmlNode *inputTermNode;
       TeiHandlerInit(docInput);
       inputKW=TeiFindKeywords(inputScheme, inputType, NULL, inputLang);
       if (inputKW)
	 {
	   outputKW=TeiFindKeywords(outputScheme, outputType, NULL, outputLang);

	   SxmlReset(inputKW);
	   while ((inputTermNode=SxmlNextNode(inputKW)))
	     {
	       char *inputTerm;
	       char *outputTerm;
	       inputTerm=SxmlLeafText(inputTermNode);
	       outputTerm=StrSearch(matchingTable, inputTerm);
	       if (outputTerm)
		 {
		   if (!outputKW)
		     {
		       SxmlAppendChild(Tei_textClass, outputKW=SxmlElementCreate("keywords"));
		       if (outputScheme)SxmlSetAttribute(outputKW,"scheme", outputScheme);
		       if (outputType)SxmlSetAttribute(outputKW,"type", outputType);
		       if (outputLang)SxmlSetAttribute(outputKW,"xml:lang", outputLang);
		     }
		   SxmlAppendChild(outputKW, SxmlLeafCreate("term", outputTerm));
		 }
	     }
	 }
       SxmlPrint (SxmlInputRecord);
     }
  exit (EXIT_SUCCESS);
}
